@@ -9,13 +9,19 @@
 #include <config.h>
 #endif
 
+#include <unistd.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
 
-#include <gio/gnetworking.h>
+#include <gio/gio.h>
 
 #include "soup-address.h"
-#include "soup.h"
-#include "soup-misc-private.h"
+#include "soup-enum-types.h"
+#include "soup-marshal.h"
+#include "soup-misc.h"
 
 /**
  * SECTION:soup-address
@@ -24,10 +30,6 @@
  * #SoupAddress represents the address of a TCP connection endpoint:
  * both the IP address and the port. (It is somewhat like an
  * object-oriented version of struct sockaddr.)
- *
- * Although #SoupAddress is still used in some libsoup API's, it
- * should not be used in new code; use GLib's #GNetworkAddress or
- * #GSocketAddress instead.
  **/
 
 enum {
@@ -36,7 +38,6 @@ enum {
 	PROP_NAME,
 	PROP_FAMILY,
 	PROP_PORT,
-	PROP_PROTOCOL,
 	PROP_PHYSICAL,
 	PROP_SOCKADDR,
 
@@ -49,10 +50,11 @@ typedef struct {
 
 	char *name, *physical;
 	guint port;
-	const char *protocol;
 
 	GMutex lock;
+	GSList *async_lookups;
 } SoupAddressPrivate;
+#define SOUP_ADDRESS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_ADDRESS, SoupAddressPrivate))
 
 /* sockaddr generic macros */
 #define SOUP_SIN(priv) ((struct sockaddr_in *)priv->sockaddr)
@@ -72,7 +74,7 @@ typedef struct {
 			     sizeof (struct in6_addr))
 
 /* sockaddr port macros */
-#define SOUP_ADDRESS_PORT_IS_VALID(port) (((gint) port) >= 0 && port <= 65535)
+#define SOUP_ADDRESS_PORT_IS_VALID(port) (port >= 0 && port <= 65535)
 #define SOUP_ADDRESS_GET_PORT(priv) \
 	(priv->sockaddr->ss_family == AF_INET ? \
 		SOUP_SIN(priv)->sin_port : \
@@ -94,40 +96,138 @@ typedef struct {
 	memcpy (SOUP_ADDRESS_GET_DATA (priv), data, length)
 
 
+static GObject *constructor (GType                  type,
+			     guint                  n_construct_properties,
+			     GObjectConstructParam *construct_properties);
+static void set_property (GObject *object, guint prop_id,
+			  const GValue *value, GParamSpec *pspec);
+static void get_property (GObject *object, guint prop_id,
+			  GValue *value, GParamSpec *pspec);
+
 static void soup_address_connectable_iface_init (GSocketConnectableIface *connectable_iface);
+static GSocketAddressEnumerator *soup_address_connectable_enumerate (GSocketConnectable *connectable);
 
 G_DEFINE_TYPE_WITH_CODE (SoupAddress, soup_address, G_TYPE_OBJECT,
-                         G_ADD_PRIVATE (SoupAddress)
 			 G_IMPLEMENT_INTERFACE (G_TYPE_SOCKET_CONNECTABLE,
 						soup_address_connectable_iface_init))
 
 static void
 soup_address_init (SoupAddress *addr)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr);
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
 	g_mutex_init (&priv->lock);
 }
 
 static void
-soup_address_finalize (GObject *object)
+finalize (GObject *object)
 {
 	SoupAddress *addr = SOUP_ADDRESS (object);
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr);
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
-	g_free (priv->sockaddr);
-	g_free (priv->name);
-	g_free (priv->physical);
+	if (priv->sockaddr)
+		g_free (priv->sockaddr);
+	if (priv->name)
+		g_free (priv->name);
+	if (priv->physical)
+		g_free (priv->physical);
 
 	g_mutex_clear (&priv->lock);
 
 	G_OBJECT_CLASS (soup_address_parent_class)->finalize (object);
 }
 
+static void
+soup_address_class_init (SoupAddressClass *address_class)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (address_class);
+
+	g_type_class_add_private (address_class, sizeof (SoupAddressPrivate));
+
+	/* virtual method override */
+	object_class->constructor = constructor;
+	object_class->finalize = finalize;
+	object_class->set_property = set_property;
+	object_class->get_property = get_property;
+
+	/* properties */
+	/**
+	 * SOUP_ADDRESS_NAME:
+	 *
+	 * Alias for the #SoupAddress:name property. (The hostname for
+	 * this address.)
+	 **/
+	g_object_class_install_property (
+		object_class, PROP_NAME,
+		g_param_spec_string (SOUP_ADDRESS_NAME,
+				     "Name",
+				     "Hostname for this address",
+				     NULL,
+				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * SOUP_ADDRESS_FAMILY:
+	 *
+	 * Alias for the #SoupAddress:family property. (The
+	 * #SoupAddressFamily for this address.)
+	 **/
+	g_object_class_install_property (
+		object_class, PROP_FAMILY,
+		g_param_spec_enum (SOUP_ADDRESS_FAMILY,
+				   "Family",
+				   "Address family for this address",
+				   SOUP_TYPE_ADDRESS_FAMILY,
+				   SOUP_ADDRESS_FAMILY_INVALID,
+				   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * SOUP_ADDRESS_PORT:
+	 *
+	 * An alias for the #SoupAddress:port property. (The port for
+	 * this address.)
+	 **/
+	g_object_class_install_property (
+		object_class, PROP_PORT,
+		g_param_spec_int (SOUP_ADDRESS_PORT,
+				  "Port",
+				  "Port for this address",
+				  -1, 65535, -1,
+				  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * SOUP_ADDRESS_PHYSICAL:
+	 *
+	 * An alias for the #SoupAddress:physical property. (The
+	 * stringified IP address for this address.)
+	 **/
+	g_object_class_install_property (
+		object_class, PROP_PHYSICAL,
+		g_param_spec_string (SOUP_ADDRESS_PHYSICAL,
+				     "Physical address",
+				     "IP address for this address",
+				     NULL,
+				     G_PARAM_READABLE));
+	/**
+	 * SOUP_ADDRESS_SOCKADDR:
+	 *
+	 * An alias for the #SoupAddress:sockaddr property. (A pointer
+	 * to the struct sockaddr for this address.)
+	 **/
+	g_object_class_install_property (
+		object_class, PROP_SOCKADDR,
+		g_param_spec_pointer (SOUP_ADDRESS_SOCKADDR,
+				      "sockaddr",
+				      "struct sockaddr for this address",
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+}
+
+static void
+soup_address_connectable_iface_init (GSocketConnectableIface *connectable_iface)
+{
+  connectable_iface->enumerate  = soup_address_connectable_enumerate;
+}
+
 static GObject *
-soup_address_constructor (GType                  type,
-			  guint                  n_construct_properties,
-			  GObjectConstructParam *construct_properties)
+constructor (GType                  type,
+	     guint                  n_construct_properties,
+	     GObjectConstructParam *construct_properties)
 {
 	GObject *addr;
 	SoupAddressPrivate *priv;
@@ -136,7 +236,7 @@ soup_address_constructor (GType                  type,
 		type, n_construct_properties, construct_properties);
 	if (!addr)
 		return NULL;
-	priv = soup_address_get_instance_private (SOUP_ADDRESS (addr));
+	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
 	if (!priv->name && !priv->sockaddr) {
 		g_object_unref (addr);
@@ -147,10 +247,10 @@ soup_address_constructor (GType                  type,
 }
 
 static void
-soup_address_set_property (GObject *object, guint prop_id,
-			   const GValue *value, GParamSpec *pspec)
+set_property (GObject *object, guint prop_id,
+	      const GValue *value, GParamSpec *pspec)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (SOUP_ADDRESS (object));
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (object);
 	SoupAddressFamily family;
 	struct sockaddr *sa;
 	int len, port;
@@ -189,10 +289,6 @@ soup_address_set_property (GObject *object, guint prop_id,
 			SOUP_ADDRESS_SET_PORT (priv, htons (port));
 		break;
 
-	case PROP_PROTOCOL:
-		priv->protocol = g_intern_string (g_value_get_string (value));
-		break;
-
 	case PROP_SOCKADDR:
 		sa = g_value_get_pointer (value);
 		if (!sa)
@@ -211,10 +307,10 @@ soup_address_set_property (GObject *object, guint prop_id,
 }
 
 static void
-soup_address_get_property (GObject *object, guint prop_id,
-			   GValue *value, GParamSpec *pspec)
+get_property (GObject *object, guint prop_id,
+	      GValue *value, GParamSpec *pspec)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (SOUP_ADDRESS (object));
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (object);
 
 	switch (prop_id) {
 	case PROP_NAME:
@@ -232,9 +328,6 @@ soup_address_get_property (GObject *object, guint prop_id,
 	case PROP_PHYSICAL:
 		g_value_set_string (value, soup_address_get_physical (SOUP_ADDRESS (object)));
 		break;
-	case PROP_PROTOCOL:
-		g_value_set_string (value, priv->protocol);
-		break;
 	case PROP_SOCKADDR:
 		g_value_set_pointer (value, priv->sockaddr);
 		break;
@@ -244,104 +337,6 @@ soup_address_get_property (GObject *object, guint prop_id,
 	}
 }
 
-static void
-soup_address_class_init (SoupAddressClass *address_class)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (address_class);
-
-	/* virtual method override */
-	object_class->constructor  = soup_address_constructor;
-	object_class->finalize     = soup_address_finalize;
-	object_class->set_property = soup_address_set_property;
-	object_class->get_property = soup_address_get_property;
-
-	/* properties */
-	/**
-	 * SOUP_ADDRESS_NAME:
-	 *
-	 * Alias for the #SoupAddress:name property. (The hostname for
-	 * this address.)
-	 **/
-	g_object_class_install_property (
-		object_class, PROP_NAME,
-		g_param_spec_string (SOUP_ADDRESS_NAME,
-				     "Name",
-				     "Hostname for this address",
-				     NULL,
-				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				     G_PARAM_STATIC_STRINGS));
-	/**
-	 * SOUP_ADDRESS_FAMILY:
-	 *
-	 * Alias for the #SoupAddress:family property. (The
-	 * #SoupAddressFamily for this address.)
-	 **/
-	g_object_class_install_property (
-		object_class, PROP_FAMILY,
-		g_param_spec_enum (SOUP_ADDRESS_FAMILY,
-				   "Family",
-				   "Address family for this address",
-				   SOUP_TYPE_ADDRESS_FAMILY,
-				   SOUP_ADDRESS_FAMILY_INVALID,
-				   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				   G_PARAM_STATIC_STRINGS));
-	/**
-	 * SOUP_ADDRESS_PORT:
-	 *
-	 * An alias for the #SoupAddress:port property. (The port for
-	 * this address.)
-	 **/
-	g_object_class_install_property (
-		object_class, PROP_PORT,
-		g_param_spec_int (SOUP_ADDRESS_PORT,
-				  "Port",
-				  "Port for this address",
-				  -1, 65535, -1,
-				  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				  G_PARAM_STATIC_STRINGS));
-	/**
-	 * SOUP_ADDRESS_PROTOCOL:
-	 *
-	 * Alias for the #SoupAddress:protocol property. (The URI scheme
-	 * used with this address.)
-	 **/
-	g_object_class_install_property (
-		object_class, PROP_PROTOCOL,
-		g_param_spec_string (SOUP_ADDRESS_PROTOCOL,
-				     "Protocol",
-				     "URI scheme for this address",
-				     NULL,
-				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				     G_PARAM_STATIC_STRINGS));
-	/**
-	 * SOUP_ADDRESS_PHYSICAL:
-	 *
-	 * An alias for the #SoupAddress:physical property. (The
-	 * stringified IP address for this address.)
-	 **/
-	g_object_class_install_property (
-		object_class, PROP_PHYSICAL,
-		g_param_spec_string (SOUP_ADDRESS_PHYSICAL,
-				     "Physical address",
-				     "IP address for this address",
-				     NULL,
-				     G_PARAM_READABLE |
-				     G_PARAM_STATIC_STRINGS));
-	/**
-	 * SOUP_ADDRESS_SOCKADDR:
-	 *
-	 * An alias for the #SoupAddress:sockaddr property. (A pointer
-	 * to the struct sockaddr for this address.)
-	 **/
-	g_object_class_install_property (
-		object_class, PROP_SOCKADDR,
-		g_param_spec_pointer (SOUP_ADDRESS_SOCKADDR,
-				      "sockaddr",
-				      "struct sockaddr for this address",
-				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				      G_PARAM_STATIC_STRINGS));
-}
-
 /**
  * soup_address_new:
  * @name: a hostname or physical address
@@ -374,7 +369,7 @@ soup_address_new (const char *name, guint port)
  * Returns a #SoupAddress equivalent to @sa (or %NULL if @sa's
  * address family isn't supported)
  *
- * Return value: (nullable): the new #SoupAddress
+ * Return value: (allow-none): the new #SoupAddress
  **/
 SoupAddress *
 soup_address_new_from_sockaddr (struct sockaddr *sa, int len)
@@ -388,17 +383,6 @@ soup_address_new_from_sockaddr (struct sockaddr *sa, int len)
 			     NULL);
 }
 
-SoupAddress *
-soup_address_new_from_gsockaddr (GSocketAddress *addr)
-{
-	struct sockaddr_storage sa;
-
-	g_socket_address_to_native (addr, &sa, sizeof (sa), NULL);
-	return g_object_new (SOUP_TYPE_ADDRESS,
-			     SOUP_ADDRESS_SOCKADDR, &sa,
-			     NULL);
-}
-
 /**
  * SoupAddressFamily:
  * @SOUP_ADDRESS_FAMILY_INVALID: an invalid %SoupAddress
@@ -424,7 +408,7 @@ soup_address_new_from_gsockaddr (GSocketAddress *addr)
  * for @family (or %NULL if @family isn't supported), suitable for
  * using as a listening #SoupSocket.
  *
- * Return value: (nullable): the new #SoupAddress
+ * Return value: (allow-none): the new #SoupAddress
  **/
 SoupAddress *
 soup_address_new_any (SoupAddressFamily family, guint port)
@@ -449,23 +433,20 @@ soup_address_new_any (SoupAddressFamily family, guint port)
  * soup_address_is_resolved() to safely test whether or not an address
  * is resolved before fetching its name or address.
  *
- * Return value: (nullable): the hostname, or %NULL if it is not known.
+ * Return value: (allow-none): the hostname, or %NULL if it is not known.
  **/
 const char *
 soup_address_get_name (SoupAddress *addr)
 {
-	SoupAddressPrivate *priv;
-
 	g_return_val_if_fail (SOUP_IS_ADDRESS (addr), NULL);
-	priv = soup_address_get_instance_private (addr);
 
-	return priv->name;
+	return SOUP_ADDRESS_GET_PRIVATE (addr)->name;
 }
 
 /**
  * soup_address_get_sockaddr:
  * @addr: a #SoupAddress
- * @len: (out): return location for sockaddr length
+ * @len: return location for sockaddr length
  *
  * Returns the sockaddr associated with @addr, with its length in
  * *@len. If the sockaddr is not yet known, returns %NULL.
@@ -475,7 +456,7 @@ soup_address_get_name (SoupAddress *addr)
  * soup_address_is_resolved() to safely test whether or not an address
  * is resolved before fetching its name or address.
  *
- * Return value: (nullable) (transfer none): the sockaddr, or %NULL
+ * Return value: (allow-none) (transfer none): the sockaddr, or %NULL
  **/
 struct sockaddr *
 soup_address_get_sockaddr (SoupAddress *addr, int *len)
@@ -483,7 +464,7 @@ soup_address_get_sockaddr (SoupAddress *addr, int *len)
 	SoupAddressPrivate *priv;
 
 	g_return_val_if_fail (SOUP_IS_ADDRESS (addr), NULL);
-	priv = soup_address_get_instance_private (addr);
+	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
 	if (priv->sockaddr && len)
 		*len = SOUP_ADDRESS_FAMILY_SOCKADDR_SIZE (SOUP_ADDRESS_GET_FAMILY (priv));
@@ -504,7 +485,7 @@ soup_address_get_sockaddr (SoupAddress *addr, int *len)
 GSocketAddress *
 soup_address_get_gsockaddr (SoupAddress *addr)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr);
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
 	return g_socket_address_new_from_native (priv->sockaddr,
 						 SOUP_ADDRESS_FAMILY_SOCKADDR_SIZE (SOUP_ADDRESS_GET_FAMILY (priv)));
@@ -513,7 +494,7 @@ soup_address_get_gsockaddr (SoupAddress *addr)
 static GInetAddress *
 soup_address_make_inet_address (SoupAddress *addr)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr);
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 	GSocketAddress *gsa;
 	GInetAddress *gia;
 
@@ -537,7 +518,7 @@ soup_address_make_inet_address (SoupAddress *addr)
  * soup_address_is_resolved() to safely test whether or not an address
  * is resolved before fetching its name or address.
  *
- * Return value: (nullable): the physical address, or %NULL
+ * Return value: (allow-none): the physical address, or %NULL
  **/
 const char *
 soup_address_get_physical (SoupAddress *addr)
@@ -545,7 +526,7 @@ soup_address_get_physical (SoupAddress *addr)
 	SoupAddressPrivate *priv;
 
 	g_return_val_if_fail (SOUP_IS_ADDRESS (addr), NULL);
-	priv = soup_address_get_instance_private (addr);
+	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
 	if (!priv->sockaddr)
 		return NULL;
@@ -572,72 +553,16 @@ soup_address_get_physical (SoupAddress *addr)
 guint
 soup_address_get_port (SoupAddress *addr)
 {
-	SoupAddressPrivate *priv;
-
 	g_return_val_if_fail (SOUP_IS_ADDRESS (addr), 0);
-	priv = soup_address_get_instance_private (addr);
 
-	return priv->port;
-}
-
-
-/* Tries to resolve priv->name as an IP address, possibly including an
- * IPv6 scope id.
- */
-static void
-maybe_resolve_ip (SoupAddress *addr)
-{
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr);
-	const char *pct, *ip;
-	char *tmp = NULL;
-	GSocketConnectable *gaddr;
-	GSocketAddressEnumerator *sa_enum;
-	GSocketAddress *saddr;
-
-	if (priv->sockaddr || !priv->name)
-		return;
-
-	pct = strchr (priv->name, '%');
-	if (pct)
-		ip = tmp = g_strndup (priv->name, pct - priv->name);
-	else
-		ip = priv->name;
-
-	if (!g_hostname_is_ip_address (ip)) {
-		g_free (tmp);
-		return;
-	}
-	g_free (tmp);
-
-	gaddr = g_network_address_new (priv->name, priv->port);
-	if (!gaddr)
-		return;
-
-	sa_enum = g_socket_connectable_enumerate (gaddr);
-	saddr = g_socket_address_enumerator_next (sa_enum, NULL, NULL);
-	if (saddr) {
-		priv->n_addrs = 1;
-		priv->sockaddr = g_new (struct sockaddr_storage, 1);
-		if (!g_socket_address_to_native (saddr, priv->sockaddr,
-						 sizeof (struct sockaddr_storage),
-						 NULL)) {
-			/* can't happen: We know the address format is supported
-			 * and the buffer is large enough
-			 */
-			g_warn_if_reached ();
-		}
-		g_object_unref (saddr);
-	}
-
-	g_object_unref (sa_enum);
-	g_object_unref (gaddr);
+	return SOUP_ADDRESS_GET_PRIVATE (addr)->port;
 }
 
 
 static guint
 update_addrs (SoupAddress *addr, GList *addrs, GError *error)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr);
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 	GInetAddress *gia;
 	GSocketAddress *gsa;
 	int i;
@@ -676,7 +601,7 @@ update_addrs (SoupAddress *addr, GList *addrs, GError *error)
 static guint
 update_name (SoupAddress *addr, const char *name, GError *error)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr);
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
 	if (error) {
 		if (error->domain == G_IO_ERROR &&
@@ -694,48 +619,56 @@ update_name (SoupAddress *addr, const char *name, GError *error)
 }
 
 typedef struct {
-	SoupAddress         *addr;
 	SoupAddressCallback  callback;
 	gpointer             callback_data;
 } SoupAddressResolveAsyncData;
 
 static void
-complete_resolve_async (SoupAddressResolveAsyncData *res_data, guint status)
+complete_resolve_async (SoupAddress *addr, guint status)
 {
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
+	SoupAddressResolveAsyncData *res_data;
+	GSList *lookups, *l;
 	GSource *current_source;
 	GMainContext *current_context;
 
-	if (res_data->callback) {
-		/* Awful hack; to make soup_socket_connect_async()
-		 * with an non-default async_context work correctly,
-		 * we need to ensure that the non-default context
-		 * (which we're now running in) is the thread-default
-		 * when the callbacks are run...
-		 */
-		current_source = g_main_current_source ();
-		if (current_source && !g_source_is_destroyed (current_source))
-			current_context = g_source_get_context (current_source);
-		else
-			current_context = NULL;
-		g_main_context_push_thread_default (current_context);
+	lookups = priv->async_lookups;
+	priv->async_lookups = NULL;
 
-		res_data->callback (res_data->addr, status,
-				    res_data->callback_data);
+	/* Awful hack; to make soup_socket_connect_async() with an
+	 * non-default async_context work correctly, we need to ensure
+	 * that the non-default context (which we're now running in)
+	 * is the thread-default when the callbacks are run...
+	 */
+	current_source = g_main_current_source ();
+	if (current_source && !g_source_is_destroyed (current_source))
+		current_context = g_source_get_context (current_source);
+	else
+		current_context = NULL;
+	g_main_context_push_thread_default (current_context);
 
-		g_main_context_pop_thread_default (current_context);
+	for (l = lookups; l; l = l->next) {
+		res_data = l->data;
+
+		if (res_data->callback) {
+			res_data->callback (addr, status,
+					    res_data->callback_data);
+		}
+		g_slice_free (SoupAddressResolveAsyncData, res_data);
 	}
+	g_slist_free (lookups);
 
-	g_object_unref (res_data->addr);
-	g_slice_free (SoupAddressResolveAsyncData, res_data);
+	g_main_context_pop_thread_default (current_context);
+
+	g_object_unref (addr);
 }
 
 static void
 lookup_resolved (GObject *source, GAsyncResult *result, gpointer user_data)
 {
 	GResolver *resolver = G_RESOLVER (source);
-	SoupAddressResolveAsyncData *res_data = user_data;
-	SoupAddress *addr = res_data->addr;
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr);
+	SoupAddress *addr = user_data;
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 	GError *error = NULL;
 	guint status;
 
@@ -760,7 +693,7 @@ lookup_resolved (GObject *source, GAsyncResult *result, gpointer user_data)
 	g_object_ref (addr);
 	g_object_set_data (G_OBJECT (addr), "async-resolved-error", error);
 
-	complete_resolve_async (res_data, status);
+	complete_resolve_async (addr, status);
 
 	g_object_set_data (G_OBJECT (addr), "async-resolved-error", NULL);
 	g_object_unref (addr);
@@ -769,9 +702,9 @@ lookup_resolved (GObject *source, GAsyncResult *result, gpointer user_data)
 }
 
 static gboolean
-idle_complete_resolve (gpointer res_data)
+idle_complete_resolve (gpointer addr)
 {
-	complete_resolve_async (res_data, SOUP_STATUS_OK);
+	complete_resolve_async (addr, SOUP_STATUS_OK);
 	return FALSE;
 }
 
@@ -817,58 +750,63 @@ soup_address_resolve_async (SoupAddress *addr, GMainContext *async_context,
 	SoupAddressPrivate *priv;
 	SoupAddressResolveAsyncData *res_data;
 	GResolver *resolver;
+	gboolean already_started;
 
 	g_return_if_fail (SOUP_IS_ADDRESS (addr));
-	priv = soup_address_get_instance_private (addr);
+	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 	g_return_if_fail (priv->name || priv->sockaddr);
 
 	/* We don't need to do locking here because the async case is
 	 * not intended to be thread-safe.
 	 */
 
-	if (priv->name && !priv->sockaddr)
-		maybe_resolve_ip (addr);
 	if (priv->name && priv->sockaddr && !callback)
 		return;
 
 	res_data = g_slice_new0 (SoupAddressResolveAsyncData);
-	res_data->addr = g_object_ref (addr);
 	res_data->callback = callback;
 	res_data->callback_data = user_data;
 
+	already_started = priv->async_lookups != NULL;
+	priv->async_lookups = g_slist_prepend (priv->async_lookups, res_data);
+
+	if (already_started)
+		return;
+
+	g_object_ref (addr);
+
+	if (priv->name && priv->sockaddr) {
+		soup_add_completion (async_context, idle_complete_resolve, addr);
+		return;
+	}
+
+	resolver = g_resolver_get_default ();
 	if (async_context)
 		g_main_context_push_thread_default (async_context);
 
-	if (priv->name && priv->sockaddr)
-		soup_add_completion (async_context, idle_complete_resolve, res_data);
-	else {
-		resolver = g_resolver_get_default ();
+	if (priv->name) {
+		g_resolver_lookup_by_name_async (resolver, priv->name,
+						 cancellable,
+						 lookup_resolved, addr);
+	} else {
+		GInetAddress *gia;
 
-		if (priv->name) {
-			g_resolver_lookup_by_name_async (resolver, priv->name,
-							 cancellable,
-							 lookup_resolved, res_data);
-		} else {
-			GInetAddress *gia;
-
-			gia = soup_address_make_inet_address (addr);
-			g_resolver_lookup_by_address_async (resolver, gia,
-							    cancellable,
-							    lookup_resolved, res_data);
-			g_object_unref (gia);
-		}
-
-		g_object_unref (resolver);
+		gia = soup_address_make_inet_address (addr);
+		g_resolver_lookup_by_address_async (resolver, gia,
+						    cancellable,
+						    lookup_resolved, addr);
+		g_object_unref (gia);
 	}
 
 	if (async_context)
 		g_main_context_pop_thread_default (async_context);
+	g_object_unref (resolver);
 }
 
 static guint
 resolve_sync_internal (SoupAddress *addr, GCancellable *cancellable, GError **error)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr);
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 	GResolver *resolver;
 	guint status;
 	GError *my_err = NULL;
@@ -882,10 +820,6 @@ resolve_sync_internal (SoupAddress *addr, GCancellable *cancellable, GError **er
 	 * blocking op, and then re-lock it to modify @addr.
 	 */
 	g_mutex_lock (&priv->lock);
-
-	if (priv->name && !priv->sockaddr)
-		maybe_resolve_ip (addr);
-
 	if (!priv->sockaddr) {
 		GList *addrs;
 
@@ -911,7 +845,6 @@ resolve_sync_internal (SoupAddress *addr, GCancellable *cancellable, GError **er
 		g_free (name);
 	} else
 		status = SOUP_STATUS_OK;
-
 	g_mutex_unlock (&priv->lock);
 
 	if (my_err)
@@ -947,7 +880,7 @@ soup_address_resolve_sync (SoupAddress *addr, GCancellable *cancellable)
 	SoupAddressPrivate *priv;
 
 	g_return_val_if_fail (SOUP_IS_ADDRESS (addr), SOUP_STATUS_MALFORMED);
-	priv = soup_address_get_instance_private (addr);
+	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 	g_return_val_if_fail (priv->name || priv->sockaddr, SOUP_STATUS_MALFORMED);
 
 	return resolve_sync_internal (addr, cancellable, NULL);
@@ -970,7 +903,7 @@ soup_address_is_resolved (SoupAddress *addr)
 	gboolean resolved;
 
 	g_return_val_if_fail (SOUP_IS_ADDRESS (addr), FALSE);
-	priv = soup_address_get_instance_private (addr);
+	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
 	g_mutex_lock (&priv->lock);
 	resolved = priv->sockaddr && priv->name;
@@ -993,7 +926,7 @@ soup_address_is_resolved (SoupAddress *addr)
 guint
 soup_address_hash_by_name (gconstpointer addr)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (SOUP_ADDRESS (addr));
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
 	g_return_val_if_fail (priv->name != NULL, 0);
 	return g_str_hash (priv->name);
@@ -1034,8 +967,8 @@ soup_address_hash_by_name (gconstpointer addr)
 gboolean
 soup_address_equal_by_name (gconstpointer addr1, gconstpointer addr2)
 {
-	SoupAddressPrivate *priv1 = soup_address_get_instance_private (SOUP_ADDRESS (addr1));
-	SoupAddressPrivate *priv2 = soup_address_get_instance_private (SOUP_ADDRESS (addr2));
+	SoupAddressPrivate *priv1 = SOUP_ADDRESS_GET_PRIVATE (addr1);
+	SoupAddressPrivate *priv2 = SOUP_ADDRESS_GET_PRIVATE (addr2);
 
 	g_return_val_if_fail (priv1->name != NULL, FALSE);
 	g_return_val_if_fail (priv2->name != NULL, FALSE);
@@ -1056,7 +989,7 @@ soup_address_equal_by_name (gconstpointer addr1, gconstpointer addr2)
 guint
 soup_address_hash_by_ip (gconstpointer addr)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (SOUP_ADDRESS (addr));
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 	guint hash;
 
 	g_return_val_if_fail (priv->sockaddr != NULL, 0);
@@ -1094,8 +1027,8 @@ soup_address_hash_by_ip (gconstpointer addr)
 gboolean
 soup_address_equal_by_ip (gconstpointer addr1, gconstpointer addr2)
 {
-	SoupAddressPrivate *priv1 = soup_address_get_instance_private (SOUP_ADDRESS (addr1));
-	SoupAddressPrivate *priv2 = soup_address_get_instance_private (SOUP_ADDRESS (addr2));
+	SoupAddressPrivate *priv1 = SOUP_ADDRESS_GET_PRIVATE (addr1);
+	SoupAddressPrivate *priv2 = SOUP_ADDRESS_GET_PRIVATE (addr2);
 	int size;
 
 	g_return_val_if_fail (priv1->sockaddr != NULL, FALSE);
@@ -1141,7 +1074,7 @@ soup_address_address_enumerator_finalize (GObject *object)
 static GSocketAddress *
 next_address (SoupAddressAddressEnumerator *addr_enum)
 {
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr_enum->addr);
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr_enum->addr);
 	struct sockaddr_storage *ss;
 	int next_addr;
 
@@ -1169,7 +1102,7 @@ soup_address_address_enumerator_next (GSocketAddressEnumerator  *enumerator,
 {
 	SoupAddressAddressEnumerator *addr_enum =
 		SOUP_ADDRESS_ADDRESS_ENUMERATOR (enumerator);
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr_enum->addr);
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr_enum->addr);
 
 	if (!priv->sockaddr) {
 		if (resolve_sync_internal (addr_enum->addr, cancellable, error) != SOUP_STATUS_OK)
@@ -1182,19 +1115,15 @@ soup_address_address_enumerator_next (GSocketAddressEnumerator  *enumerator,
 static void
 got_addresses (SoupAddress *addr, guint status, gpointer user_data)
 {
-	GTask *task = user_data;
+	GSimpleAsyncResult *simple = user_data;
 	GError *error;
 
 	error = g_object_get_data (G_OBJECT (addr), "async-resolved-error");
 	if (error)
-		g_task_return_error (task, g_error_copy (error));
-	else {
-		GSocketAddress *socket_addr;
+		g_simple_async_result_set_from_error (simple, error);
 
-		socket_addr = next_address (g_task_get_source_object (task));
-		g_task_return_pointer (task, socket_addr, g_object_unref);
-	}
-	g_object_unref (task);
+	g_simple_async_result_complete (simple);
+	g_object_unref (simple);
 }
 
 static void
@@ -1205,17 +1134,19 @@ soup_address_address_enumerator_next_async (GSocketAddressEnumerator  *enumerato
 {
 	SoupAddressAddressEnumerator *addr_enum =
 		SOUP_ADDRESS_ADDRESS_ENUMERATOR (enumerator);
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr_enum->addr);
-	GTask *task;
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr_enum->addr);
+	GSimpleAsyncResult *simple;
+
+	simple = g_simple_async_result_new (G_OBJECT (enumerator),
+					    callback, user_data,
+					    soup_address_address_enumerator_next_async);
 
-	task = g_task_new (enumerator, cancellable, callback, user_data);
 	if (!priv->sockaddr) {
-		soup_address_resolve_async (addr_enum->addr,
-					    g_main_context_get_thread_default (),
-					    cancellable, got_addresses, task);
+		soup_address_resolve_async (addr_enum->addr, NULL, cancellable,
+					    got_addresses, simple);
 	} else {
-		g_task_return_pointer (task, next_address (addr_enum), g_object_unref);
-		g_object_unref (task);
+		g_simple_async_result_complete_in_idle (simple);
+		g_object_unref (simple);
 	}
 }
 
@@ -1224,7 +1155,14 @@ soup_address_address_enumerator_next_finish (GSocketAddressEnumerator  *enumerat
 					     GAsyncResult              *result,
 					     GError                   **error)
 {
-	return g_task_propagate_pointer (G_TASK (result), error);
+	SoupAddressAddressEnumerator *addr_enum =
+		SOUP_ADDRESS_ADDRESS_ENUMERATOR (enumerator);
+	GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+
+	if (g_simple_async_result_propagate_error (simple, error))
+		return NULL;
+	else 
+		return next_address (addr_enum);
 }
 
 static void
@@ -1252,46 +1190,10 @@ soup_address_connectable_enumerate (GSocketConnectable *connectable)
 	SoupAddressPrivate *priv;
 
 	addr_enum = g_object_new (SOUP_TYPE_ADDRESS_ADDRESS_ENUMERATOR, NULL);
-	addr_enum->addr = g_object_ref (SOUP_ADDRESS (connectable));
+	addr_enum->addr = g_object_ref (connectable);
 
-	priv = soup_address_get_instance_private (addr_enum->addr);
+	priv = SOUP_ADDRESS_GET_PRIVATE (addr_enum->addr);
 	addr_enum->orig_offset = priv->offset;
 
 	return (GSocketAddressEnumerator *)addr_enum;
 }
-
-static GSocketAddressEnumerator *
-soup_address_connectable_proxy_enumerate (GSocketConnectable *connectable)
-{
-	SoupAddress *addr = SOUP_ADDRESS (connectable);
-	SoupAddressPrivate *priv = soup_address_get_instance_private (addr);
-	GSocketAddressEnumerator *proxy_enum;
-	SoupURI *uri;
-	char *uri_string;
-
-	/* We cheerily assume "http" here because you shouldn't be
-	 * using SoupAddress any more if you're not doing HTTP anyway.
-	 */
-	uri = soup_uri_new (NULL);
-	soup_uri_set_scheme (uri, priv->protocol ? priv->protocol : "http");
-	soup_uri_set_host (uri, priv->name ? priv->name : soup_address_get_physical (addr));
-	soup_uri_set_port (uri, priv->port);
-	soup_uri_set_path (uri, "");
-	uri_string = soup_uri_to_string_internal (uri, FALSE, FALSE, TRUE);
-
-	proxy_enum = g_object_new (G_TYPE_PROXY_ADDRESS_ENUMERATOR,
-				   "connectable", connectable,
-				   "uri", uri_string,
-				   NULL);
-	g_free (uri_string);
-	soup_uri_free (uri);
-
-	return proxy_enum;
-}
-
-static void
-soup_address_connectable_iface_init (GSocketConnectableIface *connectable_iface)
-{
-  connectable_iface->enumerate       = soup_address_connectable_enumerate;
-  connectable_iface->proxy_enumerate = soup_address_connectable_proxy_enumerate;
-}
