@@ -108,7 +108,8 @@ finalize (GObject *object)
 		g_object_unref (priv->tunnel_addr);
 	if (priv->proxy_uri)
 		soup_uri_free (priv->proxy_uri);
-
+	if (priv->tlsdb)
+		g_object_unref (priv->tlsdb);
 	if (priv->async_context)
 		g_main_context_unref (priv->async_context);
 
@@ -167,7 +168,7 @@ soup_connection_class_init (SoupConnectionClass *connection_class)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (SoupConnectionClass, disconnected),
 			      NULL, NULL,
-			      soup_marshal_NONE__NONE,
+			      _soup_marshal_NONE__NONE,
 			      G_TYPE_NONE, 0);
 
 	/* properties */
@@ -841,11 +842,16 @@ soup_connection_disconnect (SoupConnection *conn)
 		soup_connection_set_state (conn, SOUP_CONNECTION_DISCONNECTED);
 
 	if (priv->socket) {
-		g_signal_handlers_disconnect_by_func (priv->socket,
-						      socket_disconnected, conn);
-		soup_socket_disconnect (priv->socket);
-		g_object_unref (priv->socket);
+		/* Set the socket to NULL at the beginning to avoid reentrancy
+		 * issues. soup_socket_disconnect() could trigger a reentrant
+		 * call unref'ing and disconnecting the socket twice.
+		 */
+		SoupSocket *socket = priv->socket;
 		priv->socket = NULL;
+		g_signal_handlers_disconnect_by_func (socket,
+						      socket_disconnected, conn);
+		soup_socket_disconnect (socket);
+		g_object_unref (socket);
 	}
 
 	if (old_state != SOUP_CONNECTION_DISCONNECTED)
