@@ -9,16 +9,21 @@
 #include <config.h>
 #endif
 
+#include <stdio.h>
 #include <string.h>
 
+#include "soup-cookie.h"
 #include "soup-cookie-jar.h"
-#include "soup-message-private.h"
-#include "soup-misc-private.h"
-#include "soup.h"
+#include "soup-date.h"
+#include "soup-enum-types.h"
+#include "soup-marshal.h"
+#include "soup-message.h"
+#include "soup-session-feature.h"
+#include "soup-uri.h"
 
 /**
  * SECTION:soup-cookie-jar
- * @short_description: Automatic cookie handling for SoupSession
+ * @short_description: Automatic cookie handling for #SoupSession
  *
  * A #SoupCookieJar stores #SoupCookie<!-- -->s and arrange for them
  * to be sent with the appropriate #SoupMessage<!-- -->s.
@@ -30,6 +35,18 @@
  * of long-term cookie persistence.
  **/
 
+static void soup_cookie_jar_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
+static void request_queued (SoupSessionFeature *feature, SoupSession *session,
+			    SoupMessage *msg);
+static void request_started (SoupSessionFeature *feature, SoupSession *session,
+			     SoupMessage *msg, SoupSocket *socket);
+static void request_unqueued (SoupSessionFeature *feature, SoupSession *session,
+			      SoupMessage *msg);
+
+G_DEFINE_TYPE_WITH_CODE (SoupCookieJar, soup_cookie_jar, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (SOUP_TYPE_SESSION_FEATURE,
+						soup_cookie_jar_session_feature_init))
+
 enum {
 	CHANGED,
 	LAST_SIGNAL
@@ -52,18 +69,17 @@ typedef struct {
 	guint serial;
 	SoupCookieJarAcceptPolicy accept_policy;
 } SoupCookieJarPrivate;
+#define SOUP_COOKIE_JAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_COOKIE_JAR, SoupCookieJarPrivate))
 
-static void soup_cookie_jar_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
-
-G_DEFINE_TYPE_WITH_CODE (SoupCookieJar, soup_cookie_jar, G_TYPE_OBJECT,
-                         G_ADD_PRIVATE (SoupCookieJar)
-			 G_IMPLEMENT_INTERFACE (SOUP_TYPE_SESSION_FEATURE,
-						soup_cookie_jar_session_feature_init))
+static void set_property (GObject *object, guint prop_id,
+			  const GValue *value, GParamSpec *pspec);
+static void get_property (GObject *object, guint prop_id,
+			  GValue *value, GParamSpec *pspec);
 
 static void
 soup_cookie_jar_init (SoupCookieJar *jar)
 {
-	SoupCookieJarPrivate *priv = soup_cookie_jar_get_instance_private (jar);
+	SoupCookieJarPrivate *priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
 
 	priv->domains = g_hash_table_new_full (soup_str_case_hash,
 					       soup_str_case_equal,
@@ -73,19 +89,17 @@ soup_cookie_jar_init (SoupCookieJar *jar)
 }
 
 static void
-soup_cookie_jar_constructed (GObject *object)
+constructed (GObject *object)
 {
-	SoupCookieJarPrivate *priv =
-		soup_cookie_jar_get_instance_private (SOUP_COOKIE_JAR (object));
+	SoupCookieJarPrivate *priv = SOUP_COOKIE_JAR_GET_PRIVATE (object);
 
 	priv->constructed = TRUE;
 }
 
 static void
-soup_cookie_jar_finalize (GObject *object)
+finalize (GObject *object)
 {
-	SoupCookieJarPrivate *priv =
-		soup_cookie_jar_get_instance_private (SOUP_COOKIE_JAR (object));
+	SoupCookieJarPrivate *priv = SOUP_COOKIE_JAR_GET_PRIVATE (object);
 	GHashTableIter iter;
 	gpointer key, value;
 
@@ -98,66 +112,20 @@ soup_cookie_jar_finalize (GObject *object)
 	G_OBJECT_CLASS (soup_cookie_jar_parent_class)->finalize (object);
 }
 
-static void
-soup_cookie_jar_set_property (GObject *object, guint prop_id,
-			      const GValue *value, GParamSpec *pspec)
-{
-	SoupCookieJarPrivate *priv =
-		soup_cookie_jar_get_instance_private (SOUP_COOKIE_JAR (object));
-
-	switch (prop_id) {
-	case PROP_READ_ONLY:
-		priv->read_only = g_value_get_boolean (value);
-		break;
-	case PROP_ACCEPT_POLICY:
-		priv->accept_policy = g_value_get_enum (value);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-static void
-soup_cookie_jar_get_property (GObject *object, guint prop_id,
-			      GValue *value, GParamSpec *pspec)
-{
-	SoupCookieJarPrivate *priv =
-		soup_cookie_jar_get_instance_private (SOUP_COOKIE_JAR (object));
-
-	switch (prop_id) {
-	case PROP_READ_ONLY:
-		g_value_set_boolean (value, priv->read_only);
-		break;
-	case PROP_ACCEPT_POLICY:
-		g_value_set_enum (value, priv->accept_policy);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-static gboolean
-soup_cookie_jar_real_is_persistent (SoupCookieJar *jar)
-{
-	return FALSE;
-}
-
 static void
 soup_cookie_jar_class_init (SoupCookieJarClass *jar_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (jar_class);
 
-	object_class->constructed = soup_cookie_jar_constructed;
-	object_class->finalize = soup_cookie_jar_finalize;
-	object_class->set_property = soup_cookie_jar_set_property;
-	object_class->get_property = soup_cookie_jar_get_property;
+	g_type_class_add_private (jar_class, sizeof (SoupCookieJarPrivate));
 
-	jar_class->is_persistent = soup_cookie_jar_real_is_persistent;
+	object_class->constructed = constructed;
+	object_class->finalize = finalize;
+	object_class->set_property = set_property;
+	object_class->get_property = get_property;
 
 	/**
-	 * SoupCookieJar::changed:
+	 * SoupCookieJar::changed
 	 * @jar: the #SoupCookieJar
 	 * @old_cookie: the old #SoupCookie value
 	 * @new_cookie: the new #SoupCookie value
@@ -176,7 +144,7 @@ soup_cookie_jar_class_init (SoupCookieJarClass *jar_class)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (SoupCookieJarClass, changed),
 			      NULL, NULL,
-			      NULL,
+			      _soup_marshal_NONE__BOXED_BOXED,
 			      G_TYPE_NONE, 2, 
 			      SOUP_TYPE_COOKIE | G_SIGNAL_TYPE_STATIC_SCOPE,
 			      SOUP_TYPE_COOKIE | G_SIGNAL_TYPE_STATIC_SCOPE);
@@ -193,8 +161,7 @@ soup_cookie_jar_class_init (SoupCookieJarClass *jar_class)
 				      "Read-only",
 				      "Whether or not the cookie jar is read-only",
 				      FALSE,
-				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				      G_PARAM_STATIC_STRINGS));
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	/**
 	 * SOUP_COOKIE_JAR_ACCEPT_POLICY:
@@ -217,8 +184,56 @@ soup_cookie_jar_class_init (SoupCookieJarClass *jar_class)
 				   "The policy the jar should follow to accept or reject cookies",
 				   SOUP_TYPE_COOKIE_JAR_ACCEPT_POLICY,
 				   SOUP_COOKIE_JAR_ACCEPT_ALWAYS,
-				   G_PARAM_READWRITE |
-				   G_PARAM_STATIC_STRINGS));
+				   G_PARAM_READWRITE));
+}
+
+static void
+soup_cookie_jar_session_feature_init (SoupSessionFeatureInterface *feature_interface,
+				      gpointer interface_data)
+{
+	feature_interface->request_queued = request_queued;
+	feature_interface->request_started = request_started;
+	feature_interface->request_unqueued = request_unqueued;
+}
+
+static void
+set_property (GObject *object, guint prop_id,
+	      const GValue *value, GParamSpec *pspec)
+{
+	SoupCookieJarPrivate *priv =
+		SOUP_COOKIE_JAR_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_READ_ONLY:
+		priv->read_only = g_value_get_boolean (value);
+		break;
+	case PROP_ACCEPT_POLICY:
+		priv->accept_policy = g_value_get_enum (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+get_property (GObject *object, guint prop_id,
+	      GValue *value, GParamSpec *pspec)
+{
+	SoupCookieJarPrivate *priv =
+		SOUP_COOKIE_JAR_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_READ_ONLY:
+		g_value_set_boolean (value, priv->read_only);
+		break;
+	case PROP_ACCEPT_POLICY:
+		g_value_set_enum (value, priv->accept_policy);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 /**
@@ -244,10 +259,6 @@ soup_cookie_jar_new (void)
  * This function exists for backward compatibility, but does not do
  * anything any more; cookie jars are saved automatically when they
  * are changed.
- *
- * Since: 2.24
- *
- * Deprecated: This is a no-op.
  */
 void
 soup_cookie_jar_save (SoupCookieJar *jar)
@@ -259,7 +270,7 @@ static void
 soup_cookie_jar_changed (SoupCookieJar *jar,
 			 SoupCookie *old, SoupCookie *new)
 {
-	SoupCookieJarPrivate *priv = soup_cookie_jar_get_instance_private (jar);
+	SoupCookieJarPrivate *priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
 
 	if (old && old != new)
 		g_hash_table_remove (priv->serials, old);
@@ -279,7 +290,7 @@ compare_cookies (gconstpointer a, gconstpointer b, gpointer jar)
 {
 	SoupCookie *ca = (SoupCookie *)a;
 	SoupCookie *cb = (SoupCookie *)b;
-	SoupCookieJarPrivate *priv = soup_cookie_jar_get_instance_private (jar);
+	SoupCookieJarPrivate *priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
 	int alen, blen;
 	guint aserial, bserial;
 
@@ -300,51 +311,43 @@ compare_cookies (gconstpointer a, gconstpointer b, gpointer jar)
 	return aserial - bserial;
 }
 
-static gboolean
-cookie_is_valid_for_same_site_policy (SoupCookie *cookie,
-                                      gboolean    is_safe_method,
-                                      SoupURI    *uri,
-                                      SoupURI    *top_level,
-                                      SoupURI    *cookie_uri,
-                                      gboolean    is_top_level_navigation,
-                                      gboolean    for_http)
-{
-	SoupSameSitePolicy policy = soup_cookie_get_same_site_policy (cookie);
-
-	if (policy == SOUP_SAME_SITE_POLICY_NONE)
-		return TRUE;
-
-	if (top_level == NULL)
-		return TRUE;
-
-	if (policy == SOUP_SAME_SITE_POLICY_LAX && is_top_level_navigation &&
-	    (is_safe_method || for_http == FALSE))
-		return TRUE;
-
-	if (is_top_level_navigation && cookie_uri == NULL)
-		return FALSE;
-
-	return soup_host_matches_host (soup_uri_get_host (cookie_uri ? cookie_uri : top_level), soup_uri_get_host (uri));
-}
-
-static GSList *
-get_cookies (SoupCookieJar *jar,
-             SoupURI       *uri,
-             SoupURI       *top_level,
-             SoupURI       *site_for_cookies,
-             gboolean       is_safe_method,
-             gboolean       for_http,
-             gboolean       is_top_level_navigation,
-             gboolean       copy_cookies)
+/**
+ * soup_cookie_jar_get_cookies:
+ * @jar: a #SoupCookieJar
+ * @uri: a #SoupURI
+ * @for_http: whether or not the return value is being passed directly
+ * to an HTTP operation
+ *
+ * Retrieves (in Cookie-header form) the list of cookies that would
+ * be sent with a request to @uri.
+ *
+ * If @for_http is %TRUE, the return value will include cookies marked
+ * "HttpOnly" (that is, cookies that the server wishes to keep hidden
+ * from client-side scripting operations such as the JavaScript
+ * document.cookies property). Since #SoupCookieJar sets the Cookie
+ * header itself when making the actual HTTP request, you should
+ * almost certainly be setting @for_http to %FALSE if you are calling
+ * this.
+ *
+ * Return value: the cookies, in string form, or %NULL if there are no
+ * cookies for @uri.
+ *
+ * Since: 2.24
+ **/
+char *
+soup_cookie_jar_get_cookies (SoupCookieJar *jar, SoupURI *uri,
+			     gboolean for_http)
 {
 	SoupCookieJarPrivate *priv;
 	GSList *cookies, *domain_cookies;
-	char *domain, *cur, *next_domain;
+	char *domain, *cur, *next_domain, *result;
 	GSList *new_head, *cookies_to_remove = NULL, *p;
 
-	priv = soup_cookie_jar_get_instance_private (jar);
+	g_return_val_if_fail (SOUP_IS_COOKIE_JAR (jar), NULL);
+	priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
+	g_return_val_if_fail (uri != NULL, NULL);
 
-	if (!uri->host || !uri->host[0])
+	if (!uri->host)
 		return NULL;
 
 	/* The logic here is a little weird, but the plan is that if
@@ -369,11 +372,8 @@ get_cookies (SoupCookieJar *jar,
 						     g_strdup (cur),
 						     new_head);
 			} else if (soup_cookie_applies_to_uri (cookie, uri) &&
-			           cookie_is_valid_for_same_site_policy (cookie, is_safe_method, uri, top_level,
-				                                         site_for_cookies, is_top_level_navigation,
-									 for_http) &&
 				   (for_http || !cookie->http_only))
-				cookies = g_slist_append (cookies, copy_cookies ? soup_cookie_copy (cookie) : cookie);
+				cookies = g_slist_append (cookies, cookie);
 
 			domain_cookies = next;
 		}
@@ -391,45 +391,9 @@ get_cookies (SoupCookieJar *jar,
 	}
 	g_slist_free (cookies_to_remove);
 
-	return g_slist_sort_with_data (cookies, compare_cookies, jar);
-}
-
-/**
- * soup_cookie_jar_get_cookies:
- * @jar: a #SoupCookieJar
- * @uri: a #SoupURI
- * @for_http: whether or not the return value is being passed directly
- * to an HTTP operation
- *
- * Retrieves (in Cookie-header form) the list of cookies that would
- * be sent with a request to @uri.
- *
- * If @for_http is %TRUE, the return value will include cookies marked
- * "HttpOnly" (that is, cookies that the server wishes to keep hidden
- * from client-side scripting operations such as the JavaScript
- * document.cookies property). Since #SoupCookieJar sets the Cookie
- * header itself when making the actual HTTP request, you should
- * almost certainly be setting @for_http to %FALSE if you are calling
- * this.
- *
- * Return value: (nullable): the cookies, in string form, or %NULL if
- * there are no cookies for @uri.
- *
- * Since: 2.24
- **/
-char *
-soup_cookie_jar_get_cookies (SoupCookieJar *jar, SoupURI *uri,
-			     gboolean for_http)
-{
-	GSList *cookies;
-
-	g_return_val_if_fail (SOUP_IS_COOKIE_JAR (jar), NULL);
-	g_return_val_if_fail (uri != NULL, NULL);
-
-	cookies = get_cookies (jar, uri, NULL, NULL, TRUE, for_http, FALSE, FALSE);
-
 	if (cookies) {
-		char *result = soup_cookies_to_cookie_header (cookies);
+		cookies = g_slist_sort_with_data (cookies, compare_cookies, jar);
+		result = soup_cookies_to_cookie_header (cookies);
 		g_slist_free (cookies);
 
 		if (!*result) {
@@ -442,129 +406,20 @@ soup_cookie_jar_get_cookies (SoupCookieJar *jar, SoupURI *uri,
 }
 
 /**
- * soup_cookie_jar_get_cookie_list:
+ * soup_cookie_jar_add_cookie:
  * @jar: a #SoupCookieJar
- * @uri: a #SoupURI
- * @for_http: whether or not the return value is being passed directly
- * to an HTTP operation
- *
- * Retrieves the list of cookies that would be sent with a request to @uri
- * as a #GSList of #SoupCookie objects.
- *
- * If @for_http is %TRUE, the return value will include cookies marked
- * "HttpOnly" (that is, cookies that the server wishes to keep hidden
- * from client-side scripting operations such as the JavaScript
- * document.cookies property). Since #SoupCookieJar sets the Cookie
- * header itself when making the actual HTTP request, you should
- * almost certainly be setting @for_http to %FALSE if you are calling
- * this.
- *
- * Return value: (transfer full) (element-type Soup.Cookie): a #GSList
- * with the cookies in the @jar that would be sent with a request to @uri.
- *
- * Since: 2.40
- **/
-GSList *
-soup_cookie_jar_get_cookie_list (SoupCookieJar *jar, SoupURI *uri, gboolean for_http)
-{
-	g_return_val_if_fail (SOUP_IS_COOKIE_JAR (jar), NULL);
-	g_return_val_if_fail (uri != NULL, NULL);
-
-	return get_cookies (jar, uri, NULL, NULL, TRUE, for_http, FALSE, TRUE);
-}
-
-/**
- * soup_cookie_jar_get_cookie_list_with_same_site_info:
- * @jar: a #SoupCookieJar
- * @uri: a #SoupURI
- * @top_level: (nullable): a #SoupURI for the top level document
- * @site_for_cookies: (nullable): a #SoupURI indicating the origin to get cookies for
- * @for_http: whether or not the return value is being passed directly
- * to an HTTP operation
- * @is_safe_method: if the HTTP method is safe, as defined by RFC 7231, ignored when @for_http is %FALSE
- * @is_top_level_navigation: whether or not the HTTP request is part of
- * top level navigation
- *
- * This is an extended version of soup_cookie_jar_get_cookie_list() that
- * provides more information required to use SameSite cookies. See the
- * [SameSite cookies spec](https://tools.ietf.org/html/draft-ietf-httpbis-cookie-same-site-00)
- * for more detailed information.
- *
- * Return value: (transfer full) (element-type Soup.Cookie): a #GSList
- * with the cookies in the @jar that would be sent with a request to @uri.
- *
- * Since: 2.70
- */
-GSList *
-soup_cookie_jar_get_cookie_list_with_same_site_info (SoupCookieJar *jar,
-                                                     SoupURI       *uri,
-                                                     SoupURI       *top_level,
-                                                     SoupURI       *site_for_cookies,
-                                                     gboolean       for_http,
-                                                     gboolean       is_safe_method,
-                                                     gboolean       is_top_level_navigation)
-{
-	g_return_val_if_fail (SOUP_IS_COOKIE_JAR (jar), NULL);
-	g_return_val_if_fail (uri != NULL, NULL);
-
-	return get_cookies (jar,  uri, top_level, site_for_cookies, is_safe_method, for_http, is_top_level_navigation, TRUE);
-}
-
-static const char *
-normalize_cookie_domain (const char *domain)
-{
-	/* Trim any leading dot if present to transform the cookie
-         * domain into a valid hostname.
-         */
-	if (domain != NULL && domain[0] == '.')
-		return domain + 1;
-	return domain;
-}
-
-static gboolean
-incoming_cookie_is_third_party (SoupCookie *cookie, SoupURI *first_party)
-{
-	const char *normalized_cookie_domain;
-	const char *cookie_base_domain;
-	const char *first_party_base_domain;
-
-	if (first_party == NULL || first_party->host == NULL)
-		return TRUE;
-
-	normalized_cookie_domain = normalize_cookie_domain (cookie->domain);
-	cookie_base_domain = soup_tld_get_base_domain (normalized_cookie_domain, NULL);
-	if (cookie_base_domain == NULL)
-		cookie_base_domain = cookie->domain;
-
-	first_party_base_domain = soup_tld_get_base_domain (first_party->host, NULL);
-	if (first_party_base_domain == NULL)
-		first_party_base_domain = first_party->host;
-	return !soup_host_matches_host (cookie_base_domain, first_party_base_domain);
-}
-
-/**
- * soup_cookie_jar_add_cookie_full:
- * @jar: a #SoupCookieJar
- * @cookie: (transfer full): a #SoupCookie
- * @uri: (nullable): the URI setting the cookie
- * @first_party: (nullable): the URI for the main document
+ * @cookie: a #SoupCookie
  *
  * Adds @cookie to @jar, emitting the 'changed' signal if we are modifying
  * an existing cookie or adding a valid new cookie ('valid' means
  * that the cookie's expire date is not in the past).
  *
- * @first_party will be used to reject cookies coming from third party
- * resources in case such a security policy is set in the @jar.
- *
- * @uri will be used to reject setting or overwriting secure cookies
- * from insecure origins. %NULL is treated as secure.
- * 
  * @cookie will be 'stolen' by the jar, so don't free it afterwards.
  *
- * Since: 2.68
+ * Since: 2.26
  **/
 void
-soup_cookie_jar_add_cookie_full (SoupCookieJar *jar, SoupCookie *cookie, SoupURI *uri, SoupURI *first_party)
+soup_cookie_jar_add_cookie (SoupCookieJar *jar, SoupCookie *cookie)
 {
 	SoupCookieJarPrivate *priv;
 	GSList *old_cookies, *oc, *last = NULL;
@@ -573,41 +428,13 @@ soup_cookie_jar_add_cookie_full (SoupCookieJar *jar, SoupCookie *cookie, SoupURI
 	g_return_if_fail (SOUP_IS_COOKIE_JAR (jar));
 	g_return_if_fail (cookie != NULL);
 
-	/* Never accept cookies for public domains. */
-	if (!g_hostname_is_ip_address (cookie->domain) &&
-	    soup_tld_domain_is_public_suffix (cookie->domain)) {
-		soup_cookie_free (cookie);
-		return;
-	}
-
-	priv = soup_cookie_jar_get_instance_private (jar);
-
-	if (first_party != NULL) {
-		if (priv->accept_policy == SOUP_COOKIE_JAR_ACCEPT_NEVER ||
-                    (priv->accept_policy == SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY &&
-		     incoming_cookie_is_third_party (cookie, first_party))) {
-			soup_cookie_free (cookie);
-			return;
-		}
-	}
-
-	/* Cannot set a secure cookie over http */
-	if (uri != NULL && !soup_uri_is_https (uri, NULL) && soup_cookie_get_secure (cookie)) {
-		soup_cookie_free (cookie);
-		return;
-	}
-
+	priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
 	old_cookies = g_hash_table_lookup (priv->domains, cookie->domain);
 	for (oc = old_cookies; oc; oc = oc->next) {
 		old_cookie = oc->data;
 		if (!strcmp (cookie->name, old_cookie->name) &&
 		    !g_strcmp0 (cookie->path, old_cookie->path)) {
-			if (soup_cookie_get_secure (oc->data) && uri != NULL && !soup_uri_is_https (uri, NULL)) {
-				/* We do not allow overwriting secure cookies from an insecure origin
-				 * https://tools.ietf.org/html/draft-ietf-httpbis-cookie-alone-01
-				 */
-				soup_cookie_free (cookie);
-			} else if (cookie->expires && soup_date_is_past (cookie->expires)) {
+			if (cookie->expires && soup_date_is_past (cookie->expires)) {
 				/* The new cookie has an expired date,
 				 * this is the way the the server has
 				 * of telling us that we have to
@@ -648,53 +475,6 @@ soup_cookie_jar_add_cookie_full (SoupCookieJar *jar, SoupCookie *cookie, SoupURI
 	soup_cookie_jar_changed (jar, NULL, cookie);
 }
 
-/**
- * soup_cookie_jar_add_cookie:
- * @jar: a #SoupCookieJar
- * @cookie: (transfer full): a #SoupCookie
- *
- * Adds @cookie to @jar, emitting the 'changed' signal if we are modifying
- * an existing cookie or adding a valid new cookie ('valid' means
- * that the cookie's expire date is not in the past).
- *
- * @cookie will be 'stolen' by the jar, so don't free it afterwards.
- *
- * Since: 2.26
- **/
-void
-soup_cookie_jar_add_cookie (SoupCookieJar *jar, SoupCookie *cookie)
-{
-	soup_cookie_jar_add_cookie_full (jar, cookie, NULL, NULL);
-}
-
-/**
- * soup_cookie_jar_add_cookie_with_first_party:
- * @jar: a #SoupCookieJar
- * @first_party: the URI for the main document
- * @cookie: (transfer full): a #SoupCookie
- *
- * Adds @cookie to @jar, emitting the 'changed' signal if we are modifying
- * an existing cookie or adding a valid new cookie ('valid' means
- * that the cookie's expire date is not in the past).
- *
- * @first_party will be used to reject cookies coming from third party
- * resources in case such a security policy is set in the @jar.
- *
- * @cookie will be 'stolen' by the jar, so don't free it afterwards.
- *
- * For secure cookies to work properly you may want to use
- * soup_cookie_jar_add_cookie_full().
- *
- * Since: 2.40
- **/
-void
-soup_cookie_jar_add_cookie_with_first_party (SoupCookieJar *jar, SoupURI *first_party, SoupCookie *cookie)
-{
-	g_return_if_fail (first_party != NULL);
-
-	soup_cookie_jar_add_cookie_full (jar, cookie, NULL, first_party);
-}
-
 /**
  * soup_cookie_jar_set_cookie:
  * @jar: a #SoupCookieJar
@@ -726,7 +506,7 @@ soup_cookie_jar_set_cookie (SoupCookieJar *jar, SoupURI *uri,
 	if (!uri->host)
 		return;
 
-	priv = soup_cookie_jar_get_instance_private (jar);
+	priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
 	if (priv->accept_policy == SOUP_COOKIE_JAR_ACCEPT_NEVER)
 		return;
 
@@ -735,7 +515,7 @@ soup_cookie_jar_set_cookie (SoupCookieJar *jar, SoupURI *uri,
 	soup_cookie = soup_cookie_parse (cookie, uri);
 	if (soup_cookie) {
 		/* will steal or free soup_cookie */
-		soup_cookie_jar_add_cookie_full (jar, soup_cookie, uri, NULL);
+		soup_cookie_jar_add_cookie (jar, soup_cookie);
 	}
 }
 
@@ -760,6 +540,7 @@ soup_cookie_jar_set_cookie_with_first_party (SoupCookieJar *jar,
 					     const char *cookie)
 {
 	SoupCookie *soup_cookie;
+	SoupCookieJarPrivate *priv;
 
 	g_return_if_fail (SOUP_IS_COOKIE_JAR (jar));
 	g_return_if_fail (uri != NULL);
@@ -769,9 +550,19 @@ soup_cookie_jar_set_cookie_with_first_party (SoupCookieJar *jar,
 	if (!uri->host)
 		return;
 
+	priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
+	if (priv->accept_policy == SOUP_COOKIE_JAR_ACCEPT_NEVER)
+		return;
+
 	soup_cookie = soup_cookie_parse (cookie, uri);
 	if (soup_cookie) {
-		soup_cookie_jar_add_cookie_full (jar, soup_cookie, uri, first_party);
+		if (priv->accept_policy == SOUP_COOKIE_JAR_ACCEPT_ALWAYS ||
+		    soup_cookie_domain_matches (soup_cookie, first_party->host)) {
+			/* will steal or free soup_cookie */
+			soup_cookie_jar_add_cookie (jar, soup_cookie);
+		} else {
+			soup_cookie_free (soup_cookie);
+		}
 	}
 }
 
@@ -779,79 +570,60 @@ static void
 process_set_cookie_header (SoupMessage *msg, gpointer user_data)
 {
 	SoupCookieJar *jar = user_data;
-	SoupCookieJarPrivate *priv = soup_cookie_jar_get_instance_private (jar);
+	SoupCookieJarPrivate *priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
 	GSList *new_cookies, *nc;
-	SoupURI *first_party, *uri;
 
 	if (priv->accept_policy == SOUP_COOKIE_JAR_ACCEPT_NEVER)
 		return;
 
 	new_cookies = soup_cookies_from_response (msg);
-	first_party = soup_message_get_first_party (msg);
-	uri = soup_message_get_uri (msg);
-	for (nc = new_cookies; nc; nc = nc->next) {		
-		soup_cookie_jar_add_cookie_full (jar, g_steal_pointer (&nc->data), uri, first_party);
+	for (nc = new_cookies; nc; nc = nc->next) {
+		SoupURI *first_party = soup_message_get_first_party (msg);
+		
+		if ((priv->accept_policy == SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY &&
+		     first_party != NULL && first_party->host &&
+		     soup_cookie_domain_matches (nc->data, first_party->host)) ||
+		    priv->accept_policy == SOUP_COOKIE_JAR_ACCEPT_ALWAYS)
+			soup_cookie_jar_add_cookie (jar, nc->data);
+		else
+			soup_cookie_free (nc->data);
 	}
 	g_slist_free (new_cookies);
 }
 
 static void
-msg_starting_cb (SoupMessage *msg, gpointer feature)
+request_queued (SoupSessionFeature *feature, SoupSession *session,
+		SoupMessage *msg)
 {
-	SoupCookieJar *jar = SOUP_COOKIE_JAR (feature);
-	GSList *cookies;
-
-	cookies = soup_cookie_jar_get_cookie_list_with_same_site_info (jar, soup_message_get_uri (msg),
-	                                                               soup_message_get_first_party (msg),
-							               soup_message_get_site_for_cookies (msg),
-								       TRUE,
-							               SOUP_METHOD_IS_SAFE (msg->method),
-							               soup_message_get_is_top_level_navigation (msg));
-	if (cookies != NULL) {
-		char *cookie_header = soup_cookies_to_cookie_header (cookies);
-		soup_message_headers_replace (msg->request_headers, "Cookie", cookie_header);
-		g_free (cookie_header);
-		g_slist_free_full (cookies, (GDestroyNotify)soup_cookie_free);
-	} else {
-		soup_message_headers_remove (msg->request_headers, "Cookie");
-	}
-}
-
-static void
-soup_cookie_jar_request_queued (SoupSessionFeature *feature,
-				SoupSession *session,
-				SoupMessage *msg)
-{
-	g_signal_connect (msg, "starting",
-			  G_CALLBACK (msg_starting_cb),
-			  feature);
-
 	soup_message_add_header_handler (msg, "got-headers",
 					 "Set-Cookie",
 					 G_CALLBACK (process_set_cookie_header),
 					 feature);
-        soup_message_add_status_code_handler (msg, "got-informational",
-                                              SOUP_STATUS_SWITCHING_PROTOCOLS,
-                                              G_CALLBACK (process_set_cookie_header),
-                                              feature);
 }
 
 static void
-soup_cookie_jar_request_unqueued (SoupSessionFeature *feature,
-				  SoupSession *session,
-				  SoupMessage *msg)
+request_started (SoupSessionFeature *feature, SoupSession *session,
+		 SoupMessage *msg, SoupSocket *socket)
+{
+	SoupCookieJar *jar = SOUP_COOKIE_JAR (feature);
+	char *cookies;
+
+	cookies = soup_cookie_jar_get_cookies (jar, soup_message_get_uri (msg), TRUE);
+	if (cookies) {
+		soup_message_headers_replace (msg->request_headers,
+					      "Cookie", cookies);
+		g_free (cookies);
+	} else
+		soup_message_headers_remove (msg->request_headers, "Cookie");
+}
+
+static void
+request_unqueued (SoupSessionFeature *feature, SoupSession *session,
+		  SoupMessage *msg)
 {
 	g_signal_handlers_disconnect_by_func (msg, process_set_cookie_header, feature);
 }
 
-static void
-soup_cookie_jar_session_feature_init (SoupSessionFeatureInterface *feature_interface,
-				      gpointer interface_data)
-{
-	feature_interface->request_queued = soup_cookie_jar_request_queued;
-	feature_interface->request_unqueued = soup_cookie_jar_request_unqueued;
-}
-
 /**
  * soup_cookie_jar_all_cookies:
  * @jar: a #SoupCookieJar
@@ -875,7 +647,7 @@ soup_cookie_jar_all_cookies (SoupCookieJar *jar)
 
 	g_return_val_if_fail (SOUP_IS_COOKIE_JAR (jar), NULL);
 
-	priv = soup_cookie_jar_get_instance_private (jar);
+	priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
 
 	g_hash_table_iter_init (&iter, priv->domains);
 
@@ -903,13 +675,16 @@ soup_cookie_jar_delete_cookie (SoupCookieJar *jar,
 {
 	SoupCookieJarPrivate *priv;
 	GSList *cookies, *p;
+	char *domain;
 
 	g_return_if_fail (SOUP_IS_COOKIE_JAR (jar));
 	g_return_if_fail (cookie != NULL);
 
-	priv = soup_cookie_jar_get_instance_private (jar);
+	priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
 
-	cookies = g_hash_table_lookup (priv->domains, cookie->domain);
+	domain = g_strdup (cookie->domain);
+
+	cookies = g_hash_table_lookup (priv->domains, domain);
 	if (cookies == NULL)
 		return;
 
@@ -918,7 +693,7 @@ soup_cookie_jar_delete_cookie (SoupCookieJar *jar,
 		if (soup_cookie_equal (cookie, c)) {
 			cookies = g_slist_delete_link (cookies, p);
 			g_hash_table_insert (priv->domains,
-					     g_strdup (cookie->domain),
+					     domain,
 					     cookies);
 			soup_cookie_jar_changed (jar, c, NULL);
 			soup_cookie_free (c);
@@ -943,9 +718,6 @@ soup_cookie_jar_delete_cookie (SoupCookieJar *jar,
  * document. If no first party is set in a message when this policy is
  * in effect, cookies will be assumed to be third party by default.
  *
- * The policy for accepting or rejecting cookies returned in
- * responses.
- *
  * Since: 2.30
  */
 
@@ -966,7 +738,7 @@ soup_cookie_jar_get_accept_policy (SoupCookieJar *jar)
 
 	g_return_val_if_fail (SOUP_IS_COOKIE_JAR (jar), SOUP_COOKIE_JAR_ACCEPT_ALWAYS);
 
-	priv = soup_cookie_jar_get_instance_private (jar);
+	priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
 	return priv->accept_policy;
 }
 
@@ -987,28 +759,10 @@ soup_cookie_jar_set_accept_policy (SoupCookieJar *jar,
 
 	g_return_if_fail (SOUP_IS_COOKIE_JAR (jar));
 
-	priv = soup_cookie_jar_get_instance_private (jar);
+	priv = SOUP_COOKIE_JAR_GET_PRIVATE (jar);
 
 	if (priv->accept_policy != policy) {
 		priv->accept_policy = policy;
 		g_object_notify (G_OBJECT (jar), SOUP_COOKIE_JAR_ACCEPT_POLICY);
 	}
 }
-
-/**
- * soup_cookie_jar_is_persistent:
- * @jar: a #SoupCookieJar
- *
- * Gets whether @jar stores cookies persistenly.
- *
- * Returns: %TRUE if @jar storage is persistent or %FALSE otherwise.
- *
- * Since: 2.40
- **/
-gboolean
-soup_cookie_jar_is_persistent (SoupCookieJar *jar)
-{
-	g_return_val_if_fail (SOUP_IS_COOKIE_JAR (jar), FALSE);
-
-	return SOUP_COOKIE_JAR_GET_CLASS (jar)->is_persistent (jar);
-}
