@@ -12,9 +12,9 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <glib.h>
 
 #include "soup-date.h"
-#include "soup.h"
 
 /**
  * SoupDate:
@@ -88,7 +88,20 @@ days_in_month (int month, int year)
 		return nonleap_days_in_month[month];
 }
 
-G_DEFINE_BOXED_TYPE (SoupDate, soup_date, soup_date_copy, soup_date_free)
+GType
+soup_date_get_type (void)
+{
+	static volatile gsize type_volatile = 0;
+
+	if (g_once_init_enter (&type_volatile)) {
+		GType type = g_boxed_type_register_static (
+			g_intern_static_string ("SoupDate"),
+			(GBoxedCopyFunc) soup_date_copy,
+			(GBoxedFreeFunc) soup_date_free);
+		g_once_init_leave (&type_volatile, type);
+	}
+	return type_volatile;
+}
 
 static void
 soup_date_fixup (SoupDate *date)
@@ -188,7 +201,7 @@ soup_date_new (int year, int month, int day,
  * offset_seconds is 0, returns the current time.
  *
  * If @offset_seconds would indicate a time not expressible as a
- * <type>time_t</type>, the return value will be clamped into range.
+ * #time_t, the return value will be clamped into range.
  *
  * Return value: a new #SoupDate
  **/
@@ -525,7 +538,7 @@ soup_date_new_from_string (const char *date_string)
 
 /**
  * soup_date_new_from_time_t:
- * @when: a <type>time_t</type>
+ * @when: a #time_t
  *
  * Creates a #SoupDate corresponding to @when
  *
@@ -680,13 +693,13 @@ soup_date_to_string (SoupDate *date, SoupDateFormat format)
  * soup_date_to_time_t:
  * @date: a #SoupDate
  *
- * Converts @date to a <type>time_t</type>.
+ * Converts @date to a %time_t.
  *
- * If @date is not representable as a <type>time_t</type>, it will be
- * clamped into range. (In particular, some HTTP cookies have
- * expiration dates after "Y2.038k" (2038-01-19T03:14:07Z).)
+ * If @date is not representable as a %time_t, it will be clamped into
+ * range. (In particular, some HTTP cookies have expiration dates
+ * after "Y2.038k" (2038-01-19T03:14:07Z).)
  *
- * Return value: @date as a <type>time_t</type>
+ * Return value: @date as a %time_t
  **/
 time_t
 soup_date_to_time_t (SoupDate *date)
@@ -897,8 +910,6 @@ soup_date_get_offset (SoupDate *date)
  * @date: a #SoupDate
  *
  * Copies @date.
- *
- * Since: 2.24
  **/
 SoupDate *
 soup_date_copy (SoupDate *date)
@@ -917,8 +928,6 @@ soup_date_copy (SoupDate *date)
  * @date: a #SoupDate
  *
  * Frees @date.
- *
- * Since: 2.24
  **/
 void
 soup_date_free (SoupDate *date)
