@@ -1,17 +1,21 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
+#include "libsoup/soup.h"
+#include "libsoup/soup-auth.h"
+#include "libsoup/soup-session.h"
 
 #include "test-utils.h"
 
-static const char *base_uri;
 static GMainLoop *loop;
 
-#if defined (G_OS_WIN32) && defined (_MSC_VER)
-#define WIN32_LEAN_AND_MEAN
-#include <windows.h>
-
-#define sleep(x) Sleep (x * 1000)
-#endif
-
 typedef struct {
 	/* Explanation of what you should see */
 	const char *explanation;
@@ -43,6 +47,10 @@ typedef struct {
 	guint final_status;
 } SoupAuthTest;
 
+/* Will either point to main_tests or relogin_tests
+ */
+static SoupAuthTest *current_tests;
+
 static SoupAuthTest main_tests[] = {
 	{ "No auth available, should fail",
 	  "Basic/realm1/", "", FALSE, "0", SOUP_STATUS_UNAUTHORIZED },
@@ -150,12 +158,7 @@ static SoupAuthTest main_tests[] = {
 	  "Digest/realm1/not/", "1", FALSE, /* should not be used */ "1", SOUP_STATUS_UNAUTHORIZED },
 
 	{ "Make sure we've forgotten it",
-	  "Digest/realm1/", "", FALSE, "0", SOUP_STATUS_UNAUTHORIZED },
-
-	{ "Fail with URI-embedded password, then use right password in the authenticate signal",
-	  "Basic/realm3/", "43", TRUE, "43", SOUP_STATUS_OK },
-
-	{ NULL }
+	  "Digest/realm1/", "", FALSE, "0", SOUP_STATUS_UNAUTHORIZED }
 };
 
 static const char *auths[] = {
@@ -211,12 +214,14 @@ handler (SoupMessage *msg, gpointer data)
 
 	if (*expected) {
 		exp = *expected - '0';
-		soup_test_assert (auth == exp,
-				  "expected %s", auths[exp]);
+		if (auth != exp) {
+			debug_printf (1, "    expected %s!\n", auths[exp]);
+			errors++;
+		}
 		memmove (expected, expected + 1, strlen (expected));
 	} else {
-		soup_test_assert (*expected,
-				  "expected to be finished");
+		debug_printf (1, "    expected to be finished\n");
+		errors++;
 	}
 }
 
@@ -224,18 +229,18 @@ static void
 authenticate (SoupSession *session, SoupMessage *msg,
 	      SoupAuth *auth, gboolean retrying, gpointer data)
 {
-	SoupAuthTest *test = data;
+	int *i = data;
 	char *username, *password;
 	char num;
 
-	if (!test->provided[0])
+	if (!current_tests[*i].provided[0])
 		return;
 	if (retrying) {
-		if (!test->provided[1])
+		if (!current_tests[*i].provided[1])
 			return;
-		num = test->provided[1];
+		num = current_tests[*i].provided[1];
 	} else
-		num = test->provided[0];
+		num = current_tests[*i].provided[0];
 
 	username = g_strdup_printf ("user%c", num);
 	password = g_strdup_printf ("realm%c", num);
@@ -251,10 +256,13 @@ bug271540_sent (SoupMessage *msg, gpointer data)
 	gboolean *authenticated = data;
 	int auth = identify_auth (msg);
 
-	soup_test_assert (*authenticated || !auth,
-			  "using auth on message %d before authenticating", n);
-	soup_test_assert (!*authenticated || auth,
-			  "sent unauthenticated message %d after authenticating", n);
+	if (!*authenticated && auth) {
+		debug_printf (1, "    using auth on message %d before authenticating!!??\n", n);
+		errors++;
+	} else if (*authenticated && !auth) {
+		debug_printf (1, "    sent unauthenticated message %d after authenticating!\n", n);
+		errors++;
+	}
 }
 
 static void
@@ -273,8 +281,8 @@ bug271540_authenticate (SoupSession *session, SoupMessage *msg,
 		soup_auth_authenticate (auth, "user1", "realm1");
 		*authenticated = TRUE;
 	} else {
-		soup_test_assert (!*authenticated,
-				  "asked to authenticate message %d after authenticating", n);
+		debug_printf (1, "    asked to authenticate message %d after authenticating!\n", n);
+		errors++;
 	}
 }
 
@@ -282,8 +290,13 @@ static void
 bug271540_finished (SoupSession *session, SoupMessage *msg, gpointer data)
 {
 	int *left = data;
+	int n = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (msg), "#"));
 
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
+	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+		debug_printf (1, "      got status '%d %s' on message %d!\n",
+			      msg->status_code, msg->reason_phrase, n);
+		errors++;
+	}
 
 	(*left)--;
 	if (!*left)
@@ -291,7 +304,7 @@ bug271540_finished (SoupSession *session, SoupMessage *msg, gpointer data)
 }
 
 static void
-do_pipelined_auth_test (void)
+do_pipelined_auth_test (const char *base_uri)
 {
 	SoupSession *session;
 	SoupMessage *msg;
@@ -299,10 +312,7 @@ do_pipelined_auth_test (void)
 	char *uri;
 	int i;
 
-	g_test_bug ("271540");
-
-	SOUP_TEST_SKIP_IF_NO_APACHE;
-
+	debug_printf (1, "Testing pipelined auth (bug 271540):\n");
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
 
 	authenticated = FALSE;
@@ -421,18 +431,13 @@ digest_nonce_unauthorized (SoupMessage *msg, gpointer data)
 
 static void
 do_digest_nonce_test (SoupSession *session,
-		      const char *nth, const char *uri, gboolean use_auth_cache,
+		      const char *nth, const char *uri,
 		      gboolean expect_401, gboolean expect_signal)
 {
 	SoupMessage *msg;
 	gboolean got_401;
 
 	msg = soup_message_new (SOUP_METHOD_GET, uri);
-	if (!use_auth_cache) {
-		SoupMessageFlags flags = soup_message_get_flags (msg);
-
-		soup_message_set_flags (msg, flags | SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE);
-	}
 	if (expect_signal) {
 		g_signal_connect (session, "authenticate",
 				  G_CALLBACK (digest_nonce_authenticate),
@@ -444,40 +449,41 @@ do_digest_nonce_test (SoupSession *session,
 					      &got_401);
 	got_401 = FALSE;
 	soup_session_send_message (session, msg);
-	soup_test_assert (got_401 == expect_401,
-			  "%s request %s a 401 Unauthorized!\n", nth,
-			  got_401 ? "got" : "did not get");
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
-
-	if (expect_signal) {
-		g_signal_handlers_disconnect_by_func (session,
-						      G_CALLBACK (digest_nonce_authenticate),
-						      NULL);
+	if (got_401 != expect_401) {
+		debug_printf (1, "  %s request %s a 401 Unauthorized!\n", nth,
+			      got_401 ? "got" : "did not get");
+		errors++;
 	}
-
+	if (msg->status_code != SOUP_STATUS_OK) {
+		debug_printf (1, "  %s request got status %d %s!\n", nth,
+			      msg->status_code, msg->reason_phrase);
+		errors++;
+	}
+	if (errors == 0)
+		debug_printf (1, "  %s request succeeded\n", nth);
 	g_object_unref (msg);
 }
 
 static void
-do_digest_expiration_test (void)
+do_digest_expiration_test (const char *base_uri)
 {
 	SoupSession *session;
 	char *uri;
 
-	SOUP_TEST_SKIP_IF_NO_APACHE;
+	debug_printf (1, "\nTesting digest nonce expiration:\n");
 
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
 
 	uri = g_strconcat (base_uri, "Digest/realm1/", NULL);
-	do_digest_nonce_test (session, "First", uri, TRUE, TRUE, TRUE);
+	do_digest_nonce_test (session, "First", uri, TRUE, TRUE);
 	g_free (uri);
 	sleep (2);
 	uri = g_strconcat (base_uri, "Digest/realm1/expire/", NULL);
-	do_digest_nonce_test (session, "Second", uri, TRUE, TRUE, FALSE);
+	do_digest_nonce_test (session, "Second", uri, TRUE, FALSE);
 	sleep (1);
-	do_digest_nonce_test (session, "Third", uri, TRUE, FALSE, FALSE);
+	do_digest_nonce_test (session, "Third", uri, FALSE, FALSE);
 	sleep (1);
-	do_digest_nonce_test (session, "Fourth", uri, TRUE, FALSE, FALSE);
+	do_digest_nonce_test (session, "Fourth", uri, FALSE, FALSE);
 	g_free (uri);
 
 	soup_test_session_abort_unref (session);
@@ -536,8 +542,10 @@ async_authenticate_assert_once (SoupSession *session, SoupMessage *msg,
 
 	debug_printf (2, "  async_authenticate_assert_once\n");
 
-	soup_test_assert (!*been_here,
-			  "async_authenticate_assert_once called twice");
+	if (*been_here) {
+		debug_printf (1, "  ERROR: async_authenticate_assert_once called twice\n");
+		errors++;
+	}
 	*been_here = TRUE;
 }
 
@@ -549,8 +557,10 @@ async_authenticate_assert_once_and_stop (SoupSession *session, SoupMessage *msg,
 
 	debug_printf (2, "  async_authenticate_assert_once_and_stop\n");
 
-	soup_test_assert (!*been_here,
-			  "async_authenticate_assert_once called twice");
+	if (*been_here) {
+		debug_printf (1, "  ERROR: async_authenticate_assert_once called twice\n");
+		errors++;
+	}
 	*been_here = TRUE;
 
 	soup_session_pause_message (session, msg);
@@ -558,7 +568,7 @@ async_authenticate_assert_once_and_stop (SoupSession *session, SoupMessage *msg,
 }
 
 static void
-do_async_auth_good_password_test (void)
+do_async_auth_test (const char *base_uri)
 {
 	SoupSession *session;
 	SoupMessage *msg1, *msg2, *msg3, msg2_bak;
@@ -566,14 +576,16 @@ do_async_auth_good_password_test (void)
 	char *uri;
 	SoupAuth *auth = NULL;
 	int remaining;
+	gboolean been_there;
 
-	SOUP_TEST_SKIP_IF_NO_APACHE;
+	debug_printf (1, "\nTesting async auth:\n");
 
 	loop = g_main_loop_new (NULL, TRUE);
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-	uri = g_strconcat (base_uri, "Basic/realm1/", NULL);
 	remaining = 0;
 
+	uri = g_strconcat (base_uri, "Basic/realm1/", NULL);
+
 	msg1 = soup_message_new ("GET", uri);
 	g_object_set_data (G_OBJECT (msg1), "id", GINT_TO_POINTER (1));
 	auth_id = g_signal_connect (session, "authenticate",
@@ -590,7 +602,13 @@ do_async_auth_good_password_test (void)
 	g_object_set_data (G_OBJECT (msg2), "id", GINT_TO_POINTER (2));
 	soup_session_send_message (session, msg2);
 
-	soup_test_assert_message_status (msg2, SOUP_STATUS_UNAUTHORIZED);
+	if (msg2->status_code == SOUP_STATUS_UNAUTHORIZED)
+		debug_printf (1, "  msg2 failed as expected\n");
+	else {
+		debug_printf (1, "  msg2 got wrong status! (%u)\n",
+			      msg2->status_code);
+		errors++;
+	}
 
 	/* msg2 should be done at this point; assuming everything is
 	 * working correctly, the session won't look at it again; we
@@ -621,11 +639,25 @@ do_async_auth_good_password_test (void)
 		g_main_loop_run (loop);
 
 		/* async_finished will quit the loop */
-	} else
-		soup_test_assert (auth, "msg1 didn't get authenticate signal");
+	} else {
+		debug_printf (1, "  msg1 didn't get authenticate signal!\n");
+		errors++;
+	}
 
-	soup_test_assert_message_status (msg1, SOUP_STATUS_OK);
-	soup_test_assert_message_status (msg3, SOUP_STATUS_OK);
+	if (msg1->status_code == SOUP_STATUS_OK)
+		debug_printf (1, "  msg1 succeeded\n");
+	else {
+		debug_printf (1, "  msg1 FAILED! (%u %s)\n",
+			      msg1->status_code, msg1->reason_phrase);
+		errors++;
+	}
+	if (msg3->status_code == SOUP_STATUS_OK)
+		debug_printf (1, "  msg3 succeeded\n");
+	else {
+		debug_printf (1, "  msg3 FAILED! (%u %s)\n",
+			      msg3->status_code, msg3->reason_phrase);
+		errors++;
+	}
 
 	soup_test_session_abort_unref (session);
 
@@ -634,46 +666,27 @@ do_async_auth_good_password_test (void)
 	memcpy (msg2, &msg2_bak, sizeof (SoupMessage));
 	g_object_unref (msg2);
 
-	g_free (uri);
-	g_main_loop_unref (loop);
-}
-
-static void
-do_async_auth_bad_password_test (void)
-{
-	SoupSession *session;
-	SoupMessage *msg;
-	guint auth_id;
-	char *uri;
-	SoupAuth *auth = NULL;
-	int remaining;
-	gboolean been_there;
-
 	/* Test that giving the wrong password doesn't cause multiple
 	 * authenticate signals the second time.
 	 */
-	g_test_bug ("522601");
+	debug_printf (1, "\nTesting async auth with wrong password (#522601):\n");
 
-	SOUP_TEST_SKIP_IF_NO_APACHE;
-
-	loop = g_main_loop_new (NULL, TRUE);
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-	uri = g_strconcat (base_uri, "Basic/realm1/", NULL);
 	remaining = 0;
 	auth = NULL;
 
-	msg = soup_message_new ("GET", uri);
-	g_object_set_data (G_OBJECT (msg), "id", GINT_TO_POINTER (1));
+	msg1 = soup_message_new ("GET", uri);
+	g_object_set_data (G_OBJECT (msg1), "id", GINT_TO_POINTER (1));
 	auth_id = g_signal_connect (session, "authenticate",
 				    G_CALLBACK (async_authenticate), &auth);
-	g_object_ref (msg);
+	g_object_ref (msg1);
 	remaining++;
-	soup_session_queue_message (session, msg, async_finished, &remaining);
+	soup_session_queue_message (session, msg1, async_finished, &remaining);
 	g_main_loop_run (loop);
 	g_signal_handler_disconnect (session, auth_id);
 	soup_auth_authenticate (auth, "user1", "wrong");
 	g_object_unref (auth);
-	soup_session_unpause_message (session, msg);
+	soup_session_unpause_message (session, msg1);
 
 	been_there = FALSE;
 	auth_id = g_signal_connect (session, "authenticate",
@@ -682,71 +695,55 @@ do_async_auth_bad_password_test (void)
 	g_main_loop_run (loop);
 	g_signal_handler_disconnect (session, auth_id);
 
-	soup_test_assert (been_there,
-			  "authenticate not emitted");
+	if (!been_there) {
+		debug_printf (1, "  authenticate not emitted?\n");
+		errors++;
+	}
 
 	soup_test_session_abort_unref (session);
-	g_object_unref (msg);
-
-	g_free (uri);
-	g_main_loop_unref (loop);
-}
-
-static void
-do_async_auth_no_password_test (void)
-{
-	SoupSession *session;
-	SoupMessage *msg;
-	guint auth_id;
-	char *uri;
-	int remaining;
-	gboolean been_there;
+	g_object_unref (msg1);
 
 	/* Test that giving no password doesn't cause multiple
 	 * authenticate signals the second time.
 	 */
-	g_test_bug ("583462");
+	debug_printf (1, "\nTesting async auth with no password (#583462):\n");
 
-	SOUP_TEST_SKIP_IF_NO_APACHE;
-
-	loop = g_main_loop_new (NULL, TRUE);
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-	uri = g_strconcat (base_uri, "Basic/realm1/", NULL);
 	remaining = 0;
 
 	/* Send a message that doesn't actually authenticate
 	 */
-	msg = soup_message_new ("GET", uri);
-	g_object_set_data (G_OBJECT (msg), "id", GINT_TO_POINTER (1));
+	msg1 = soup_message_new ("GET", uri);
+	g_object_set_data (G_OBJECT (msg1), "id", GINT_TO_POINTER (1));
 	auth_id = g_signal_connect (session, "authenticate",
 				    G_CALLBACK (async_authenticate), NULL);
-	g_object_ref (msg);
+	g_object_ref (msg1);
 	remaining++;
-	soup_session_queue_message (session, msg, async_finished, &remaining);
+	soup_session_queue_message (session, msg1, async_finished, &remaining);
 	g_main_loop_run (loop);
 	g_signal_handler_disconnect (session, auth_id);
-	soup_session_unpause_message (session, msg);
+	soup_session_unpause_message (session, msg1);
 	g_main_loop_run (loop);
-	g_object_unref(msg);
+	g_object_unref(msg1);
 
 	/* Now send a second message */
-	msg = soup_message_new ("GET", uri);
-	g_object_set_data (G_OBJECT (msg), "id", GINT_TO_POINTER (2));
-	g_object_ref (msg);
+	msg1 = soup_message_new ("GET", uri);
+	g_object_set_data (G_OBJECT (msg1), "id", GINT_TO_POINTER (2));
+	g_object_ref (msg1);
 	been_there = FALSE;
 	auth_id = g_signal_connect (session, "authenticate",
 				    G_CALLBACK (async_authenticate_assert_once_and_stop),
 				    &been_there);
 	remaining++;
-	soup_session_queue_message (session, msg, async_finished, &remaining);
+	soup_session_queue_message (session, msg1, async_finished, &remaining);
 	g_main_loop_run (loop);
-	soup_session_unpause_message (session, msg);
+	soup_session_unpause_message (session, msg1);
 
 	g_main_loop_run (loop);
 	g_signal_handler_disconnect (session, auth_id);
 
 	soup_test_session_abort_unref (session);
-	g_object_unref (msg);
+	g_object_unref (msg1);
 
 	g_free (uri);
 	g_main_loop_unref (loop);
@@ -810,27 +807,41 @@ select_auth_test_one (SoupURI *uri,
 	msg = soup_message_new_from_uri ("GET", uri);
 	soup_session_send_message (session, msg);
 
-	soup_test_assert (strcmp (sad.round[0].headers, first_headers) == 0,
-			  "Header order wrong: expected %s, got %s",
-			  first_headers, sad.round[0].headers);
-	soup_test_assert (strcmp (sad.round[0].response, first_response) == 0,
-			  "Selected auth type wrong: expected %s, got %s",
-			  first_response, sad.round[0].response);
-
-	soup_test_assert (sad.round[1].headers || !second_headers,
-			  "Expected a second round");
-	soup_test_assert (!sad.round[1].headers || second_headers,
-			  "Didn't expect a second round");
-	if (second_headers && second_response) {
-		soup_test_assert (strcmp (sad.round[1].headers, second_headers) == 0,
-				  "Second round header order wrong: expected %s, got %s\n",
-				  second_headers, sad.round[1].headers);
-		soup_test_assert (strcmp (sad.round[1].response, second_response) == 0,
-				  "Second round selected auth type wrong: expected %s, got %s\n",
-				  second_response, sad.round[1].response);
+	if (strcmp (sad.round[0].headers, first_headers) != 0) {
+		debug_printf (1, "    Header order wrong: expected %s, got %s\n",
+			      first_headers, sad.round[0].headers);
+		errors++;
+	}
+	if (strcmp (sad.round[0].response, first_response) != 0) {
+		debug_printf (1, "    Selected auth type wrong: expected %s, got %s\n",
+			      first_response, sad.round[0].response);
+		errors++;
 	}
 
-	soup_test_assert_message_status (msg, final_status);
+	if (second_headers && !sad.round[1].headers) {
+		debug_printf (1, "    Expected a second round!\n");
+		errors++;
+	} else if (!second_headers && sad.round[1].headers) {
+		debug_printf (1, "    Didn't expect a second round!\n");
+		errors++;
+	} else if (second_headers) {
+		if (strcmp (sad.round[1].headers, second_headers) != 0) {
+			debug_printf (1, "    Second round header order wrong: expected %s, got %s\n",
+				      second_headers, sad.round[1].headers);
+			errors++;
+		}
+		if (strcmp (sad.round[1].response, second_response) != 0) {
+			debug_printf (1, "    Second round selected auth type wrong: expected %s, got %s\n",
+				      second_response, sad.round[1].response);
+			errors++;
+		}
+	}
+
+	if (msg->status_code != final_status) {
+		debug_printf (1, "    Final status wrong: expected %u, got %u\n",
+			      final_status, msg->status_code);
+		errors++;
+	}
 
 	g_object_unref (msg);
 	soup_test_session_abort_unref (session);
@@ -874,7 +885,7 @@ do_select_auth_test (void)
 	SoupAuthDomain *basic_auth_domain, *digest_auth_domain;
 	SoupURI *uri;
 
-	g_test_bug ("562339");
+	debug_printf (1, "\nTesting selection among multiple auths:\n");
 
 	/* It doesn't seem to be possible to configure Apache to serve
 	 * multiple auth types for a single URL. So we have to use
@@ -882,10 +893,12 @@ do_select_auth_test (void)
 	 * side of this scenario correctly, because we test it against
 	 * curl in server-auth-test.
 	 */
-	server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
+	server = soup_test_server_new (FALSE);
 	soup_server_add_handler (server, NULL,
 				 server_callback, NULL, NULL);
-	uri = soup_test_server_get_uri (server, "http", NULL);
+
+	uri = soup_uri_new ("http://127.0.0.1/");
+	soup_uri_set_port (uri, soup_server_get_port (server));
 
 	basic_auth_domain = soup_auth_domain_basic_new (
 		SOUP_AUTH_DOMAIN_REALM, "auth-test",
@@ -1028,12 +1041,14 @@ do_auth_close_test (void)
 	SoupURI *uri;
 	AuthCloseData acd;
 
-	server = soup_test_server_new (SOUP_TEST_SERVER_DEFAULT);
+	debug_printf (1, "\nTesting auth when server times out connection:\n");
+
+	server = soup_test_server_new (FALSE);
 	soup_server_add_handler (server, NULL,
 				 server_callback, NULL, NULL);
 
-	uri = soup_test_server_get_uri (server, "http", NULL);
-	soup_uri_set_path (uri, "/close");
+	uri = soup_uri_new ("http://127.0.0.1/close");
+	soup_uri_set_port (uri, soup_server_get_port (server));
 
 	basic_auth_domain = soup_auth_domain_basic_new (
 		SOUP_AUTH_DOMAIN_REALM, "auth-test",
@@ -1054,129 +1069,18 @@ do_auth_close_test (void)
 	soup_uri_free (uri);
 	soup_session_send_message (acd.session, acd.msg);
 
-	soup_test_assert_message_status (acd.msg, SOUP_STATUS_OK);
+	if (acd.msg->status_code != SOUP_STATUS_OK) {
+		debug_printf (1, "    Final status wrong: expected %u, got %u %s\n",
+			      SOUP_STATUS_OK, acd.msg->status_code,
+			      acd.msg->reason_phrase);
+		errors++;
+	}
 
 	g_object_unref (acd.msg);
 	soup_test_session_abort_unref (acd.session);
 	soup_test_server_quit_unref (server);
 }
 
-static gboolean
-infinite_cancel (gpointer session)
-{
-	soup_session_abort (session);
-	return FALSE;
-}
-
-static void
-infinite_authenticate (SoupSession *session, SoupMessage *msg,
-		       SoupAuth *auth, gboolean retrying, gpointer data)
-{
-	soup_auth_authenticate (auth, "user", "bad");
-}
-
-static void
-do_infinite_auth_test (void)
-{
-	SoupSession *session;
-	SoupMessage *msg;
-	char *uri;
-	int timeout;
-
-	SOUP_TEST_SKIP_IF_NO_APACHE;
-
-	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-	g_signal_connect (session, "authenticate",
-			  G_CALLBACK (infinite_authenticate), NULL);
-
-	uri = g_strconcat (base_uri, "Basic/realm1/", NULL);
-	msg = soup_message_new ("GET", uri);
-	g_free (uri);
-
-	timeout = g_timeout_add (500, infinite_cancel, session);
-	g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
-			       "*stuck in infinite loop*");
-	soup_session_send_message (session, msg);
-	g_test_assert_expected_messages ();
-
-	soup_test_assert (msg->status_code != SOUP_STATUS_CANCELLED,
-			  "Got stuck in loop");
-	soup_test_assert_message_status (msg, SOUP_STATUS_UNAUTHORIZED);
-
-	g_source_remove (timeout);
-	soup_test_session_abort_unref (session);
-	g_object_unref (msg);
-}
-
-static void
-disappear_request_read (SoupServer *server, SoupMessage *msg,
-			SoupClientContext *context, gpointer user_data)
-{
-	/* Remove the WWW-Authenticate header if this was a failed attempt */
-	if (soup_message_headers_get_one (msg->request_headers, "Authorization") &&
-	    msg->status_code == SOUP_STATUS_UNAUTHORIZED)
-		soup_message_headers_remove (msg->response_headers, "WWW-Authenticate");
-}
-
-static void
-disappear_authenticate (SoupSession *session, SoupMessage *msg,
-			SoupAuth *auth, gboolean retrying, gpointer data)
-{
-	int *counter = data;
-
-	(*counter)++;
-	if (!retrying)
-		soup_auth_authenticate (auth, "user", "bad");
-}
-
-static void
-do_disappearing_auth_test (void)
-{
-	SoupServer *server;
-	SoupAuthDomain *auth_domain;
-	SoupURI *uri;
-	SoupMessage *msg;
-	SoupSession *session;
-	int counter;
-
-	g_test_bug_base ("https://bugzilla.redhat.com/");
-	g_test_bug ("916224");
-
-	server = soup_test_server_new (FALSE);
-	soup_server_add_handler (server, NULL,
-				 server_callback, NULL, NULL);
-	uri = soup_test_server_get_uri (server, "http", NULL);
-
-	auth_domain = soup_auth_domain_basic_new (
-						  SOUP_AUTH_DOMAIN_REALM, "auth-test",
-						  SOUP_AUTH_DOMAIN_ADD_PATH, "/",
-						  SOUP_AUTH_DOMAIN_BASIC_AUTH_CALLBACK, server_basic_auth_callback,
-						  NULL);
-	soup_server_add_auth_domain (server, auth_domain);
-	g_signal_connect (server, "request-read",
-			  G_CALLBACK (disappear_request_read), NULL);
-
-	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-
-	counter = 0;
-	g_signal_connect (session, "authenticate",
-			  G_CALLBACK (disappear_authenticate), &counter);
-
-	msg = soup_message_new_from_uri ("GET", uri);
-	soup_session_send_message (session, msg);
-
-	soup_test_assert (counter <= 2,
-			  "Got stuck in loop");
-	soup_test_assert_message_status (msg, SOUP_STATUS_UNAUTHORIZED);
-
-	g_object_unref (msg);
-	soup_test_session_abort_unref (session);
-
-	g_object_unref (auth_domain);
-	soup_uri_free (uri);
-	soup_test_server_quit_unref (server);
-}
-
 static SoupAuthTest relogin_tests[] = {
 	{ "Auth provided via URL, should succeed",
 	  "Basic/realm12/", "1", TRUE, "01", SOUP_STATUS_OK },
@@ -1198,39 +1102,25 @@ static SoupAuthTest relogin_tests[] = {
 
 	{ "Should fail with no auth, fail again with bad password, and give up",
 	  "Basic/realm12/", "3", FALSE, "03", SOUP_STATUS_UNAUTHORIZED },
-
-	{ NULL }
-};
-
-/* https://bugzilla.gnome.org/show_bug.cgi?id=755617 */
-static SoupAuthTest basic_root_pspace_test[] = {
-	{ "Auth provided via URL, should succeed",
-	  "BasicRoot", "1", TRUE, "01", SOUP_STATUS_OK },
-
-	{ "Parent dir should automatically reuse auth",
-	  "/", "1", FALSE, "1", SOUP_STATUS_OK },
-
-	{ NULL }
 };
 
 static void
-do_batch_tests (gconstpointer data)
+do_batch_tests (const gchar *base_uri_str, gint ntests)
 {
-	const SoupAuthTest *current_tests = data;
 	SoupSession *session;
 	SoupMessage *msg;
 	char *expected, *uristr;
-	SoupURI *base;
-	guint signal;
+	SoupURI *base_uri;
 	int i;
 
-	SOUP_TEST_SKIP_IF_NO_APACHE;
-
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-	base = soup_uri_new (base_uri);
+	g_signal_connect (session, "authenticate",
+			  G_CALLBACK (authenticate), &i);
 
-	for (i = 0; current_tests[i].url; i++) {
-		SoupURI *soup_uri = soup_uri_new_with_base (base, current_tests[i].url);
+	base_uri = soup_uri_new (base_uri_str);
+
+	for (i = 0; i < ntests; i++) {
+		SoupURI *soup_uri = soup_uri_new_with_base (base_uri, current_tests[i].url);
 
 		debug_printf (1, "Test %d: %s\n", i + 1, current_tests[i].explanation);
 
@@ -1246,7 +1136,7 @@ do_batch_tests (gconstpointer data)
 		msg = soup_message_new_from_uri (SOUP_METHOD_GET, soup_uri);
 		soup_uri_free (soup_uri);
 		if (!msg) {
-			g_printerr ("auth-test: Could not parse URI\n");
+			fprintf (stderr, "auth-test: Could not parse URI\n");
 			exit (1);
 		}
 
@@ -1261,271 +1151,62 @@ do_batch_tests (gconstpointer data)
 		soup_message_add_status_code_handler (
 			msg, "got_headers", SOUP_STATUS_OK,
 			G_CALLBACK (handler), expected);
-
-		signal = g_signal_connect (session, "authenticate",
-					   G_CALLBACK (authenticate),
-					   (gpointer)&current_tests[i]);
 		soup_session_send_message (session, msg);
-		g_signal_handler_disconnect (session, signal);
-
-		soup_test_assert_message_status (msg, current_tests[i].final_status);
-		soup_test_assert (!*expected,
-				  "expected %d more round(s)\n",
-				  (int)strlen (expected));
-
+		if (msg->status_code != SOUP_STATUS_UNAUTHORIZED &&
+		    msg->status_code != SOUP_STATUS_OK) {
+			debug_printf (1, "  %d %s !\n", msg->status_code,
+				      msg->reason_phrase);
+			errors++;
+		}
+		if (*expected) {
+			debug_printf (1, "  expected %d more round(s)\n",
+				      (int)strlen (expected));
+			errors++;
+		}
 		g_free (expected);
+
+		if (msg->status_code != current_tests[i].final_status) {
+			debug_printf (1, "  expected %d\n",
+				      current_tests[i].final_status);
+		}
+
 		debug_printf (1, "\n");
 
 		g_object_unref (msg);
 	}
-	soup_uri_free (base);
+	soup_uri_free (base_uri);
 
 	soup_test_session_abort_unref (session);
 }
 
-static void
-do_clear_credentials_test (void)
-{
-	SoupSession *session;
-	SoupAuthManager *manager;
-	char *uri;
-
-	SOUP_TEST_SKIP_IF_NO_APACHE;
-
-	session = soup_test_session_new (SOUP_TYPE_SESSION, NULL);
-
-	uri = g_strconcat (base_uri, "Digest/realm1/", NULL);
-	do_digest_nonce_test (session, "First", uri, TRUE, TRUE, TRUE);
-
-	manager = SOUP_AUTH_MANAGER (soup_session_get_feature (session, SOUP_TYPE_AUTH_MANAGER));
-	soup_auth_manager_clear_cached_credentials (manager);
-
-	do_digest_nonce_test (session, "Second", uri, TRUE, TRUE, TRUE);
-	g_free (uri);
-
-	soup_test_session_abort_unref (session);
-}
-
-static void
-do_message_do_not_use_auth_cache_test (void)
-{
-	SoupSession *session;
-	SoupAuthManager *manager;
-	SoupMessage *msg;
-	SoupMessageFlags flags;
-	SoupURI *soup_uri;
-	char *uri;
-
-	SOUP_TEST_SKIP_IF_NO_APACHE;
-
-	session = soup_test_session_new (SOUP_TYPE_SESSION, NULL);
-
-	uri = g_strconcat (base_uri, "Digest/realm1/", NULL);
-
-	/* First check that cached credentials are not used */
-	do_digest_nonce_test (session, "First", uri, TRUE, TRUE, TRUE);
-	do_digest_nonce_test (session, "Second", uri, TRUE, FALSE, FALSE);
-	do_digest_nonce_test (session, "Third", uri, FALSE, TRUE, TRUE);
-
-	/* Passing credentials in the URI should always authenticate
-	 * no matter whether the cache is used or not
-	 */
-	soup_uri = soup_uri_new (uri);
-	soup_uri_set_user (soup_uri, "user1");
-	soup_uri_set_password (soup_uri, "realm1");
-	msg = soup_message_new_from_uri (SOUP_METHOD_GET, soup_uri);
-	flags = soup_message_get_flags (msg);
-	soup_message_set_flags (msg, flags | SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE);
-	soup_session_send_message (session, msg);
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
-	g_object_unref (msg);
-	soup_uri_free (soup_uri);
-
-	manager = SOUP_AUTH_MANAGER (soup_session_get_feature (session, SOUP_TYPE_AUTH_MANAGER));
-
-	soup_auth_manager_clear_cached_credentials (manager);
-
-	/* Now check that credentials are not stored */
-	do_digest_nonce_test (session, "First", uri, FALSE, TRUE, TRUE);
-	do_digest_nonce_test (session, "Second", uri, TRUE, TRUE, TRUE);
-	do_digest_nonce_test (session, "Third", uri, TRUE, FALSE, FALSE);
-
-	/* Credentials were stored for uri, but if we set SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE flag,
-	 * and we don't have the authenticate signal, it should respond with 401
-	 */
-	msg = soup_message_new (SOUP_METHOD_GET, uri);
-	flags = soup_message_get_flags (msg);
-	soup_message_set_flags (msg, flags | SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE);
-	soup_session_send_message (session, msg);
-	soup_test_assert_message_status (msg, SOUP_STATUS_UNAUTHORIZED);
-	g_object_unref (msg);
-	g_free (uri);
-
-	soup_test_session_abort_unref (session);
-}
-
-static void
-async_no_auth_cache_authenticate (SoupSession *session, SoupMessage *msg,
-				  SoupAuth *auth, gboolean retrying, SoupAuth **auth_out)
-{
-	debug_printf (1, "  async_no_auth_cache_authenticate\n");
-
-	soup_session_pause_message (session, msg);
-	*auth_out = g_object_ref (auth);
-	g_main_loop_quit (loop);
-}
-
-static void
-async_no_auth_cache_finished (SoupSession *session, SoupMessage *msg, gpointer user_data)
-{
-	debug_printf (1, "  async_no_auth_cache_finished\n");
-
-	g_main_loop_quit (loop);
-}
-
-static void
-do_async_message_do_not_use_auth_cache_test (void)
-{
-	SoupSession *session;
-	SoupMessage *msg;
-	char *uri;
-	SoupAuth *auth = NULL;
-	SoupMessageFlags flags;
-
-	SOUP_TEST_SKIP_IF_NO_APACHE;
-
-	loop = g_main_loop_new (NULL, TRUE);
-	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-	uri = g_strconcat (base_uri, "Basic/realm1/", NULL);
-
-	msg = soup_message_new ("GET", uri);
-	g_free (uri);
-	g_signal_connect (session, "authenticate",
-			  G_CALLBACK (async_no_auth_cache_authenticate), &auth);
-	flags = soup_message_get_flags (msg);
-	soup_message_set_flags (msg, flags | SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE);
-	g_object_ref (msg);
-	soup_session_queue_message (session, msg, async_no_auth_cache_finished, NULL);
-	g_main_loop_run (loop);
-
-	soup_test_assert_message_status (msg, SOUP_STATUS_UNAUTHORIZED);
-
-	soup_test_assert (auth, "msg didn't get authenticate signal");
-	soup_auth_authenticate (auth, "user1", "realm1");
-	g_object_unref (auth);
-
-	soup_session_unpause_message (session, msg);
-	g_main_loop_run (loop);
-
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
-
-	soup_test_session_abort_unref (session);
-	g_object_unref (msg);
-	g_main_loop_unref (loop);
-}
-
-static void
-has_authorization_header_authenticate (SoupSession *session, SoupMessage *msg,
-				       SoupAuth *auth, gboolean retrying, gpointer data)
-{
-	SoupAuth **saved_auth = data;
-
-	soup_auth_authenticate (auth, "user1", "realm1");
-	*saved_auth = g_object_ref (auth);
-}
-
-static void
-has_authorization_header_authenticate_assert (SoupSession *session, SoupMessage *msg,
-					      SoupAuth *auth, gboolean retrying, gpointer data)
-{
-	soup_test_assert (FALSE, "authenticate emitted unexpectedly");
-}
-
-static void
-do_message_has_authorization_header_test (void)
-{
-	SoupSession *session;
-	SoupMessage *msg;
-	SoupAuthManager *manager;
-	SoupAuth *auth = NULL;
-	char *token;
-	guint auth_id;
-	char *uri;
-
-	g_test_bug ("775882");
-
-	SOUP_TEST_SKIP_IF_NO_APACHE;
-
-	session = soup_test_session_new (SOUP_TYPE_SESSION, NULL);
-	uri = g_strconcat (base_uri, "Digest/realm1/", NULL);
-
-	msg = soup_message_new ("GET", uri);
-	auth_id = g_signal_connect (session, "authenticate",
-			  G_CALLBACK (has_authorization_header_authenticate), &auth);
-	soup_session_send_message (session, msg);
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
-	soup_test_assert (SOUP_IS_AUTH (auth), "Expected a SoupAuth");
-	token = soup_auth_get_authorization (auth, msg);
-	g_object_unref (auth);
-	g_object_unref (msg);
-	g_signal_handler_disconnect (session, auth_id);
-
-	manager = SOUP_AUTH_MANAGER (soup_session_get_feature (session, SOUP_TYPE_AUTH_MANAGER));
-	soup_auth_manager_clear_cached_credentials (manager);
-
-	msg = soup_message_new ("GET", uri);
-	soup_message_headers_replace (msg->request_headers, "Authorization", token);
-	auth_id = g_signal_connect (session, "authenticate",
-				    G_CALLBACK (has_authorization_header_authenticate_assert),
-				    NULL);
-	soup_session_send_message (session, msg);
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
-	g_object_unref (msg);
-
-	/* Check that we can also provide our own Authorization header when not using credentials cache. */
-	soup_auth_manager_clear_cached_credentials (manager);
-	msg = soup_message_new ("GET", uri);
-	soup_message_headers_replace (msg->request_headers, "Authorization", token);
-	soup_message_set_flags (msg, soup_message_get_flags (msg) | SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE);
-	soup_session_send_message (session, msg);
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
-	g_object_unref (msg);
-	g_free (token);
-	g_signal_handler_disconnect (session, auth_id);
-
-	g_free (uri);
-	soup_test_session_abort_unref (session);
-}
-
 int
 main (int argc, char **argv)
 {
-	int ret;
+	const char *base_uri;
+	int ntests;
 
 	test_init (argc, argv, NULL);
 	apache_init ();
 
 	base_uri = "http://127.0.0.1:47524/";
 
-	g_test_add_data_func ("/auth/main-tests", main_tests, do_batch_tests);
-	g_test_add_data_func ("/auth/relogin-tests", relogin_tests, do_batch_tests);
-	g_test_add_data_func ("/auth/basic-root-pspec-test", basic_root_pspace_test, do_batch_tests);
-	g_test_add_func ("/auth/pipelined-auth", do_pipelined_auth_test);
-	g_test_add_func ("/auth/digest-expiration", do_digest_expiration_test);
-	g_test_add_func ("/auth/async-auth/good-password", do_async_auth_good_password_test);
-	g_test_add_func ("/auth/async-auth/bad-password", do_async_auth_bad_password_test);
-	g_test_add_func ("/auth/async-auth/no-password", do_async_auth_no_password_test);
-	g_test_add_func ("/auth/select-auth", do_select_auth_test);
-	g_test_add_func ("/auth/auth-close", do_auth_close_test);
-	g_test_add_func ("/auth/infinite-auth", do_infinite_auth_test);
-	g_test_add_func ("/auth/disappearing-auth", do_disappearing_auth_test);
-	g_test_add_func ("/auth/clear-credentials", do_clear_credentials_test);
-	g_test_add_func ("/auth/message-do-not-use-auth-cache", do_message_do_not_use_auth_cache_test);
-	g_test_add_func ("/auth/async-message-do-not-use-auth-cache", do_async_message_do_not_use_auth_cache_test);
-	g_test_add_func ("/auth/authorization-header-request", do_message_has_authorization_header_test);
+	/* Main tests */
+	current_tests = main_tests;
+	ntests = G_N_ELEMENTS (main_tests);
+	do_batch_tests (base_uri, ntests);
 
-	ret = g_test_run ();
+	/* Re-login tests */
+	current_tests = relogin_tests;
+	ntests = G_N_ELEMENTS (relogin_tests);
+	do_batch_tests (base_uri, ntests);
+
+	/* Other regression tests */
+	do_pipelined_auth_test (base_uri);
+	do_digest_expiration_test (base_uri);
+	do_async_auth_test (base_uri);
+	do_select_auth_test ();
+	do_auth_close_test ();
 
 	test_cleanup ();
-	return ret;
+	return errors != 0;
 }
